-- Bootstrapping code
AngeleDei = CreateFrame("Frame");
AngeleDei:SetScript('OnEvent', function(self, ...)
	self:Enable(...);
end);
AngeleDei:RegisterEvent("PLAYER_ENTERING_WORLD");

-- Local state
local queue = { };
local lastList = nil;
local hopo = 0;

-- Various visual effects
local currentShiftStart = nil;
local currentShiftEnd = nil;
local nextShiftStart = nil;
local nextShiftEnd = nil;
local otherShiftStart = nil;
local otherShiftEnd = nil;
local idleShift = nil;
local fadeInStart = nil;
local fadeInEnd = nil;
local fadeOutStart = nil;
local fadeOutEnd = nil;
local unlockEnd = nil;
local messageStart = nil;
local messageShown = nil;
local messageFadeOutStart = nil;
local messageEnd = nil;
local nextSound = GetTime();
local specLookupAfterTalentSwitch = nil;

-- Various display parameters
local QUEUE_SIZE = 4;			-- The last is always invisible
local NEXT_ICON_OFFSET = 96;
local ICON_SPACING = 36;
local FONT = "Interface\\AddOns\\AngeleDei\\fonts\\russel square lt.ttf";
local MESSAGE_FONT = "Fonts\\FRIZQT__.TTF";		-- "Fonts\\SKURRI.TTF";
local ALPHA_STEP = 1.0/(QUEUE_SIZE-1);

-- Configuration settings
local config;
local spec = nil;

-- Current mode (true = aoe, false = single-target)
local aoeMode = nil;

-- Current rotation and keystrokes
local keystrokes;

-- In-combat flag
local inCombat = UnitAffectingCombat("player");

-- Vengeance stoplight style colors
local vengeanceStoplight = { "red", "orange", "yellow", "lime", "green" };

-- Volume reduction
local sound = { "lutnia1_1.ogg", "lutnia1_1_50.ogg", "lutnia1_1_25.ogg", "lutnia1_1_12.ogg" };
local soundVolume = 1;

-- For debugging purposes
function AngeleDei:GetConfig()
	return config;
end

-- Initialize the addon
function AngeleDei:Enable(event)
	-- We needed these events for initialization only
	AngeleDei:UnregisterEvent("PLAYER_ENTERING_WORLD");
	AngeleDei:SetScript("OnEvent", nil);

	-- We'll use this frame for timer and combat log events
	AngeleDei:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	AngeleDei:RegisterEvent("PLAYER_TARGET_CHANGED");
	AngeleDei:RegisterEvent("ACTIONBAR_PAGE_CHANGED");
	AngeleDei:RegisterEvent("UNIT_AURA");
	AngeleDei:RegisterEvent("PLAYER_ALIVE");
	AngeleDei:RegisterEvent("PLAYER_DEAD");
	AngeleDei:RegisterEvent("PLAYER_REGEN_ENABLED");
	AngeleDei:RegisterEvent("PLAYER_REGEN_DISABLED");
	AngeleDei:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
	AngeleDei:RegisterEvent("GLYPH_UPDATED");
	--AngeleDei:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
	
	AngeleDei:SetScript("OnUpdate", AngeleDei.OnUpdate);
	AngeleDei:SetScript("OnEvent", AngeleDei.OnEvent);
	
	DEFAULT_CHAT_FRAME:AddMessage("|cFFFFD000Angele Dei|r: Rotations made easy."
		--.. " |TInterface\\Icons\\Ability_Paladin_DivineStorm:20:20:0:0|t"
		--.. " |TInterface\\Icons\\Ability_Paladin_ProtectoroftheInnocent:20:20:0:0|t"	
		--.. " |TInterface\\Icons\\Ability_Paladin_SwiftRetribution:20:20:0:0|t"	
		--.. " |TInterface\\Icons\\Ability_SteelMelee:20:20:0:0|t"	
		--.. " |TInterface\\Icons\\Ability_ThunderBolt:20:20:0:0|t"	
		, 1, 1, 1);
	
	-- Create a Vengeance tooltip scanner
	AngeleDei:CreateTooltipScanner();	

	-- Load configuration and bind it to the interface settings UI
	config = Config:Load();
	SpecList:LoadAllSpecs();
	Options:SetConfig(config);
	Options:SetCallback(function()
		--print("<stance check: " .. (config.checkStance and "YES" or "NO") .. ">");
		AngeleDei:LoadSpec(true);				-- In case spec detection is affected (e.g. stance check)
	end);
	
	-- Visual stuff
	AngeleDei:CreateIcons();
	
	-- Load data for the current spec
	AngeleDei:LoadSpec(true);
	
	-- Try setting the current rotation based on the action bar page (if this is not successful,
	-- the rotation will be set to the default single-target)
	--AngeleDei:RotationByActionBarPage(true);
	
	-- Display the frame (if enabled). This will also do the initial simulation run
	AngeleDei_RotationFrame:SetScale(config.scale or 1.0);
	AngeleDei_RotationFrame:EnableMouse(not config.locked);

	-- Update the holy power indicator according to the settings
	AngeleDei:UpdateHolyPowerIndicatorStyle();
	AngeleDei:UpdateVengeanceIndicatorStyle();
		
	if(spec ~= nil) then
		AngeleDei:RotationByActionBarPage(true);
		if(config.enabled) then
			AngeleDei:EnableAddon(true);
		end
	else
		AngeleDei:DisableAddon(true);
	end	

	-- Set up command line interface
	SLASH_AD1 = "/AD";
	SLASH_AD2 = "/ANGELEDEI";
	SlashCmdList["AD"] = AngeleDei_SlashCommand;
end

-- Process /ad commands
function AngeleDei_SlashCommand(msg)
	msg = string.trim(msg or "");
	
	if(msg == "cfg") then
		-- Open configuration window
		InterfaceOptionsFrame_Show();			-- This line is a hack. Other mods don't seem to need it, but I can't make the window show without it...
		InterfaceOptionsFrame_OpenToCategory('Angele Dei');
	elseif(msg == "show") or (msg == "on") then
		-- Enable the AD addon
		if(spec == nil) then
			showUnsupportedWarning();
			return;
		end
		AngeleDei:EnableAddon();
	elseif(msg == "hide") or (msg == "off") then
		-- Disable the AD addon
		if(spec == nil) then
			showUnsupportedWarning();
			return;
		end
		AngeleDei:DisableAddon();
	elseif(msg == "reset") then
		-- Re-center the frame
		if(spec == nil) then
			showUnsupportedWarning();
			return;
		end
		AngeleDei_RotationFrame:ClearAllPoints();
		AngeleDei_RotationFrame:SetPoint("CENTER", UIParent, "CENTER");
	elseif(msg == "move") then
		-- Make the window movable for 10 seconds
		if(spec == nil) then
			showUnsupportedWarning();
			return;
		end
		if(config.locked) then
			AngeleDei_RotationFrame:EnableMouse(true);
			for i=1,#queue do
				queue[i]:SetVertexColor(0, 1, 0.75);
			end
			unlockEnd = GetTime() + 10;
		end
	elseif(msg == "edit") then
		if(spec == nil) then
			showUnsupportedWarning();
			return;
		end
		RotationList:Populate(spec);
		RotationList:Show();
	else
		-- Print command line help
		for i=1,#CMDLINE_HELP do
			DEFAULT_CHAT_FRAME:AddMessage(CMDLINE_HELP[i], 1, 1, 1);
		end
	end
end

-- Enable the addon (NOOP if already enabled)
function AngeleDei:EnableAddon(force)
	if(config.enabled) and (not force) then
		return;
	end
	
	config.enabled = true;
	config:Save();

	spec.state:SetHolyPower(UnitPower("player", 9));
	spec.state:AdjustTLast(GetTime(), SIM_SHIFT, spec.spellinfo);
	AngeleDei:RunSimulation(GetTime(), "On enable", true);
	AngeleDei_RotationFrame:SetAlpha(0);
	AngeleDei_RotationFrame:Show();
	fadeInStart = GetTime();
	fadeInEnd = fadeInStart + 0.2;
end

-- Disable the addon (NOOP if already disabled)
function AngeleDei:DisableAddon(force)
	if(not config.enabled) and (not force) then
		return;
	end
	
	config.enabled = false;
	config:Save();

	fadeOutStart = GetTime();
	fadeOutEnd = fadeOutStart + 0.2;
end

-- Create Vengeance tooltip scanner
function AngeleDei:CreateTooltipScanner()
	AngeleDei_RotationFrame.scanner = CreateFrame("GameTooltip", "VengeanceStatusScanTip", nil, "GameTooltipTemplate")
	AngeleDei_RotationFrame.scanner:SetOwner(UIParent, "ANCHOR_NONE")
end

-- Create frame icons
function AngeleDei:CreateIcons()
	queue[1] = AngeleDei:createEmptyTexture(36, 1, IDLE, "");
	queue[1]:SetWidth(48);
	queue[1]:SetHeight(48);

	for i=0,QUEUE_SIZE-2 do
		queue[i+2] = AngeleDei:createEmptyTexture(NEXT_ICON_OFFSET + (i*ICON_SPACING), 1-(i+1)*ALPHA_STEP, IDLE, "");	-- i<3 and 0.3 or 0.15
	end

	-- Set textures of all icons	
	for i=1,#queue do
		queue[i]:SetTexture(spec and spec.spellinfo:GetIcon(IDLE) or "?");
		queue[i].time = 1000000000;
	end

	local y0 = (AngeleDei_RotationFrame_HP_1:GetTop() + AngeleDei_RotationFrame_HP_1:GetBottom()) / 2;
	local y1 = (AngeleDei_RotationFrame_HP_3:GetTop() + AngeleDei_RotationFrame_HP_3:GetBottom()) / 2;
	
	--local names = { "red-s", "orange-s", "yellow-s", "lime-s", "green-s" };
	local names = { "orange-s", "orange-s", "orange-s", "orange-s", "orange-s" };
	
	AngeleDei_RotationFrame.vengeanceDark = { };
	AngeleDei_RotationFrame.vengeanceLight = { };
	
	for i=0,4 do	
		local t = AngeleDei_RotationFrame:CreateTexture(nil, "BACKGROUND");
		if(i == 0) then
			t:SetPoint("CENTER", AngeleDei_RotationFrame_HP_1, "CENTER", -20, 0);
		else
			t:SetPoint("CENTER", AngeleDei_RotationFrame.vengeanceDark[0], "CENTER", 0, i/4*(y1-y0));
		end
		t:SetTexture("Interface\\AddOns\\AngeleDei\\images\\dark-s.tga");
		t:SetWidth(16);
		t:SetHeight(16);
		t:SetAlpha(1);
		
		local tt = AngeleDei_RotationFrame:CreateTexture(nil, "ARTWORK");
		tt:SetPoint("CENTER", t, "CENTER");
		
		local color = (config.vengeanceStyle == "stoplight") and vengeanceStoplight[i+1] or config.vengeanceStyle;		
		tt:SetTexture("Interface\\AddOns\\AngeleDei\\images\\" .. (color or "purple") .. "-s.tga");
		
		tt:SetWidth(16);
		tt:SetHeight(16);
		tt:SetBlendMode("BLEND");
		tt:SetAlpha(0);
		
		AngeleDei_RotationFrame.vengeanceDark[i] = t;
		AngeleDei_RotationFrame.vengeanceLight[i] = tt;

		if(config.vengeanceEnabled) then
			t:Show();
			tt:Show();
		end
	end

	if(config.vengeanceEnabled) then
		AngeleDei:UpdateVengeanceIndicator();
	end
end

-- Update the appearance of the frame based on the settings. This is called
-- after the settings are saved.
function AngeleDei:UpdateAppearance()
	keystrokes = spec:GetKeystrokes(aoeMode);
	for i=1,#queue do
		local icon = queue[i];
		icon.label:SetText(keystrokes[icon.ability]);
	end
	AngeleDei:UpdateHolyPowerIndicatorStyle();
	AngeleDei:UpdateVengeanceIndicatorStyle();
	AngeleDei_RotationFrame:SetScale(config.scale);
	AngeleDei_RotationFrame:EnableMouse(not config.locked);
	
	AngeleDei:RunSimulation(GetTime(), "On config change", true);
end

-- Start the icon shift animation
function AngeleDei:Shift(t)
	local shiftSpeed;
	if(config.shiftGCD) then
		shiftSpeed = 1.5 / spec.spellinfo:MeasureGCD();
	else
		shiftSpeed = config.shiftSpeed;
	end		

	idleShift = nil;
	currentShiftStart = t;
	currentShiftEnd = currentShiftStart + 0.3/shiftSpeed;
	nextShiftStart = t + 0.25/shiftSpeed;
	nextShiftEnd = nextShiftStart + 0.35/shiftSpeed;
	otherShiftStart = t + 0.5/shiftSpeed;
	otherShiftEnd = otherShiftStart + 0.35/shiftSpeed;

	local recycled = table.remove(queue, 1);
	queue[#queue+1] = recycled;
end

-- Update the holy power indicator based on the current UI settings
function AngeleDei:UpdateHolyPowerIndicatorStyle()
	local a1, a2, a3 = 1, 1, 1;
	
	-- Set alpha
	if(config.fadeOut) then
		if(config.style == "stoplight") then
			a1, a2, a3 = 1, 0.5, 0.5;
		else
			a1, a2, a3 = 0.5, 0.5, 1;
		end
	end
	
	AngeleDei_RotationFrame_HP_1:SetAlpha(a1);
	AngeleDei_RotationFrame_HP_2:SetAlpha(a2);
	AngeleDei_RotationFrame_HP_3:SetAlpha(a3);
	
	AngeleDei:UpdateHolyPower(getHolyPower());
	
	if(spec ~= nil) and (spec:SupportsHolyPower()) then
		AngeleDei_RotationFrame_HP_1:Show();
		AngeleDei_RotationFrame_HP_2:Show();
		AngeleDei_RotationFrame_HP_3:Show();
	else
		AngeleDei_RotationFrame_HP_1:Hide();
		AngeleDei_RotationFrame_HP_2:Hide();
		AngeleDei_RotationFrame_HP_3:Hide();
	end		
end

-- Update the vengeance indicator based on the current UI settings
function AngeleDei:UpdateVengeanceIndicatorStyle()
	for i=0,4 do
		local dark = AngeleDei_RotationFrame.vengeanceDark[i];
		local light = AngeleDei_RotationFrame.vengeanceLight[i];
		
		local color = (config.vengeanceStyle == "stoplight") and vengeanceStoplight[i+1] or config.vengeanceStyle;
		light:SetTexture("Interface\\AddOns\\AngeleDei\\images\\" .. (color or "purple") .. "-s.tga");
		
		if(config.vengeanceEnabled) then
			dark:Show();
			light:Show();
		else
			dark:Hide();
			light:Hide();
		end
	end
	
	-- Just in case we do it while we have Vengeance...
	AngeleDei:UpdateVengeanceIndicator();

	AngeleDei_RotationFrame.vengeanceDark[0]:ClearAllPoints();
	if(spec ~= nil) and (spec:SupportsHolyPower()) then
		AngeleDei_RotationFrame.vengeanceDark[0]:SetPoint("CENTER", AngeleDei_RotationFrame_HP_1, "CENTER", -20, 0);
	else
		AngeleDei_RotationFrame.vengeanceDark[0]:SetPoint("RIGHT", AngeleDei_RotationFrame_HP_1, "RIGHT", 0, 0);
	end		
end

-- Update the holy power indicator based on the current level of HoPo
function AngeleDei:UpdateHolyPower(h)
	local t1, t2, t3;
	local style = (config.style or "default");
	if(style == "stoplight") then
		t3 = (h == 1) and "red" or "dark";
		t2 = (h == 2) and "yellow" or "dark";
		t1 = (h == 3) and "green" or "dark";
	else
		t3 = (h >= 3) and style or "dark";
		t2 = (h >= 2) and style or "dark";
		t1 = (h >= 1) and style or "dark";
	end

	local a1, a2, a3 = 1, 1, 1;

	AngeleDei_RotationFrame_HP_1:SetTexture("Interface\\AddOns\\AngeleDei\\images\\" .. t1);
	AngeleDei_RotationFrame_HP_2:SetTexture("Interface\\AddOns\\AngeleDei\\images\\" .. t2);
	AngeleDei_RotationFrame_HP_3:SetTexture("Interface\\AddOns\\AngeleDei\\images\\" .. t3);
	
	-- Schedule a blink if the corresponding option is set
	if(h == 3) and (config.blink) then
		hopo3BlinkStart = GetTime();
		hopo3BlinkEnd = hopo3BlinkStart + 0.45;
	end
end

-- Handle heartbeat
function AngeleDei:OnUpdate()
	local t = GetTime();

	-- Delayed spec lookup	
	if(specLookupAfterTalentSwitch) then
		if(t >= specLookupAfterTalentSwitch) then
			specLookupAfterTalentSwitch = nil;
			AngeleDei:LoadSpec(true);
			if(spec ~= nil) then
				AngeleDei:RotationByActionBarPage(true);
				AngeleDei:UpdateHolyPowerIndicatorStyle();		-- To show or hide HoPo indicator
				AngeleDei:UpdateVengeanceIndicatorStyle();
			end
			return	
		end
	end

	-- Animate the rotation switch message
	if(messageStart) then
		local mess = AngeleDei_RotationFrame.message;
		if(t >= messageEnd) then
			mess:Hide();
			mess:SetAlpha(1);
			messageStart = nil;
			messageShown = nil;
			messageFadeOutStart = nil;
			messageEnd = nil;
		elseif(t >= messageFadeOutStart) then
			mess:SetAlpha((t - messageEnd) / (messageFadeOutStart - messageEnd));		
		elseif(t >= messageShown) then
			mess:SetAlphaGradient(100, 0);		
			mess:SetAlpha(1);
		elseif(t >= messageStart) then
			local len = string.len(mess:GetText()) + 1;
			local g = len * (t - messageStart) / (messageShown - messageStart);
			mess:SetAlphaGradient(g, 4);		
		end			
	end	
	
	-- Keep the window movable until 'unlockEnd'
	if(unlockEnd) then
		if(t >= unlockEnd) then
			for i=1,#queue do
				queue[i]:SetVertexColor(1, 1, 1);
			end
			AngeleDei_RotationFrame:EnableMouse(not config.locked);
			unlockEnd = nil;
		end
	end
	
	-- Fade entire frame in or out
	if(fadeInStart) then
		if(t >= fadeInEnd) then
			AngeleDei_RotationFrame:SetAlpha(1);
			fadeInStart = nil;
			fadeInEnd = nil;
		elseif(t >= fadeInStart) then
			AngeleDei_RotationFrame:SetAlpha((t - fadeInStart) / (fadeInEnd - fadeInStart));		
		end
	elseif(fadeOutStart) then
		if(t >= fadeOutEnd) then
			AngeleDei_RotationFrame:Hide();
			AngeleDei_RotationFrame:SetAlpha(1);
			fadeOutStart = nil;
			fadeOutEnd = nil;
		elseif(t >= fadeOutStart) then
			AngeleDei_RotationFrame:SetAlpha((t - fadeOutEnd) / (fadeOutStart - fadeOutEnd));		
		end
	end
	
	-- Update holy power indicator
	local h = getHolyPower();
	if(h ~= hopo) then
		AngeleDei:UpdateHolyPower(h);
		hopo = h;
	end

	-- Handle HoPo indicator blinking
	if(hopo3BlinkStart) then
		local hp3icon = (config.style == "stoplight") 
			and AngeleDei_RotationFrame_HP_1 
			or AngeleDei_RotationFrame_HP_3;
		local hp3texture = (config.style == "stoplight") 
			and "Interface\\AddOns\\AngeleDei\\images\\green" 
			or "Interface\\AddOns\\AngeleDei\\images\\" .. config.style;
		if(t >= hopo3BlinkEnd) then
			hopo3BlinkStart = nil;
			hopo3BlinkEnd = nil;
			hp3icon:SetTexture(hp3texture);
		else
			local a = mod(floor((t - hopo3BlinkStart)/0.075 + 0.5), 2);
			if(a == 0) then
				hp3icon:SetTexture(hp3texture);
			else
				hp3icon:SetTexture("Interface\\AddOns\\AngeleDei\\images\\dark");
			end			
		end			
	end	

	-- All icon-related animations will be disabled if the spec or class is unsupported
	if(spec == nil) then
		return;
	end

	-- Handle icon blinking
	for i=1,QUEUE_SIZE-1 do
		local x = queue[i];
		if(x.blinkStart) then
			if(t >= x.blinkEnd) then
				x.blinkStart = nil;
				x.blinkEnd = nil;
				x.previous = nil;
				
				x:SetTexture(spec.spellinfo:GetIcon(x.ability));
				x:SetVertexColor(1, 1, 1);
			elseif(t >= x.blinkStart) then
				local a = mod(floor((t - x.blinkStart)/0.075 + 0.5), 2);
				if(a == 0) then
					x:SetTexture(spec.spellinfo:GetIcon(x.previous));
					x:SetVertexColor(1, 0, 0);
				else
					x:SetTexture(spec.spellinfo:GetIcon(x.ability));
					x:SetVertexColor(1, 1, 1);
				end
			end
		end
	end

	-- A forced shift due to an idle rotation slot?
	if(idleShift) then
		if(queue[1].ability ~= IDLE) then
			--DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000Angele Dei|r: Idle shift cancelled", 1, 1, 1);
			idleShift = nil;				-- It's no longer an idle slot		
		elseif(t >= idleShift) then
			idleShift = nil;
			--DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFFAngele Dei|r: Shift (IDLE)", 1, 1, 1);
			AngeleDei:Shift(t);
			--DEFAULT_CHAT_FRAME:AddMessage("|cFF00FF80Angele Dei|r: Simulation (IDLE completed)", 1, 1, 1);
			spec.state:AdjustTLast(GetTime(), SIM_SHIFT, spec.spellinfo);
			AngeleDei:RunSimulation(t, "Idle shift");		-- Just like we would if we used an ability
		end
	end

	-- The current icon slides down and fades out
	if(currentShiftStart) then
		local last = queue[#queue];
		if(t >= currentShiftEnd) then
			currentShiftStart = nil;
			currentShiftEnd = nil;
			last:SetAlpha(0);
			last:Hide();
			last:SetWidth(32);
			last:SetHeight(32);
		elseif(t >= currentShiftStart) then
			local a = (currentShiftEnd - t)/(currentShiftEnd - currentShiftStart);
			last:SetAlpha(a);
			last.label:SetAlpha(a);
			last:SetPoint("CENTER", AngeleDei_RotationFrame, "LEFT", 36, -96*(1-a));
		end
	end
	
	-- The next icon slides left into the frame and fades in to 100%
	if(nextShiftStart) then
		if(t >= nextShiftEnd) then
			nextShiftStart = nil;
			nextShiftEnd = nil;
			queue[1]:SetAlpha(1);
			queue[1]:SetPoint("CENTER", AngeleDei_RotationFrame, "LEFT", 36, 0);
			queue[1]:SetWidth(48);
			queue[1]:SetHeight(48);
		elseif(t >= nextShiftStart) then
			local a = (t - nextShiftStart)/(nextShiftEnd - nextShiftStart);
			queue[1]:SetAlpha(1 + (a-1)*ALPHA_STEP);
			queue[1]:SetPoint("CENTER", AngeleDei_RotationFrame, "LEFT", NEXT_ICON_OFFSET - a*(NEXT_ICON_OFFSET - ICON_SPACING), 0);
			queue[1]:SetWidth(32+16*a);
			queue[1]:SetHeight(32+16*a);
		end
	end
	
	-- All other icons slide left
	if(otherShiftStart) then
		if(t >= otherShiftEnd) then
			-- This is the end of the animation. Set the final positions and alpha values.
			otherShiftStart = nil;
			otherShiftEnd = nil;

			for i=0,#queue-2 do
				queue[i+2]:SetAlpha(1-(i+1)*ALPHA_STEP); -- 0.75-i*0.25);
				queue[i+2]:SetPoint("CENTER", AngeleDei_RotationFrame, "LEFT", NEXT_ICON_OFFSET + (i*ICON_SPACING), 0);
				queue[i+2]:Show();
			end
			--DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000Angele Dei|r: Shift completed", 1, 1, 1);

			-- If the new current ability is idle, schedule another shift 0.65 = (1.5 - 'otherShiftEnd' delay) seconds later
			if(queue[1].ability == IDLE) then
				--idleShift = t + queue[1].idle - 0.85;
				idleShift = t + spec.spellinfo:GCD() - 0.85;
			end
		elseif(t >= otherShiftStart) then
			local a = (t - otherShiftStart)/(otherShiftEnd - otherShiftStart);
			for i=1,#queue-2 do
				queue[i+1]:SetAlpha(1-(i-a+1)*ALPHA_STEP); -- 0.75-(i-a)*0.25);
				queue[i+1]:SetPoint("CENTER", AngeleDei_RotationFrame, "LEFT", NEXT_ICON_OFFSET + ((i-a)*ICON_SPACING), 0);
			end
		end
	end
end

-- Get the current holy power
function getHolyPower()
	return UnitPower("player", 9);
end

-- Display the "unsupported spec" message.
function showUnsupportedWarning()
	DEFAULT_CHAT_FRAME:AddMessage("|cFFFFD000Angele Dei|r: " .. MESSAGE_UNSUPPORTED, 1, 1, 1);
end

-- Enable or disable the addon on loading or talent changes
function AngeleDei:LoadSpec(force, suppressWarning)
	local newSpec = SpecList:GetActiveSpec(config.checkStance);
	
	if(force) or (newSpec ~= spec) then
		spec = newSpec;
		if(newSpec == nil) then
			-- We do not support this spec
			keystrokes = nil;
			AngeleDei:DisableAddon(true);
			if(not suppressWarning) then
				showUnsupportedWarning();
			end
			return false;
		else
			local _, specMess = spec:GetDescription();
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFD000Angele Dei|r: " .. format(MESSAGE_SPEC, specMess), 1, 1, 1);
			spec:Load();
			keystrokes = spec:GetKeystrokes(aoeMode);
			RotationList:Populate(spec);
			-- If we are in combat already, enable the addon
			if(config.combatToggle) and (inCombat) then
				AngeleDei:EnableAddon();
			end
			-- Some aspects of the spec affect appearance of the addon
			AngeleDei:UpdateAppearance();
			return true;
		end
	end
end

-- Event handler
function AngeleDei:OnEvent(
	event, timestamp, eventType, hideCaster, 
	srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, 
	p1, p2, p3, s1, s2, s3, s4, s5, s6, s7)
	
	-- Schedule spec update on dual-spec switching
	if(event == "ACTIVE_TALENT_GROUP_CHANGED") then
		specLookupAfterTalentSwitch = GetTime() + 0.75;
		return;
	end
	
	-- Make sure we switch after the last glyph is loaded
	if(event == "GLYPH_UPDATED") then
		specLookupAfterTalentSwitch = GetTime() + 0.2;
		return;
	end
	
	-- We'll do additional handling of these two events later in the code
	-- (after we make sure we have a spec). For now we'll just keep track
	-- of the in-combat flag
	if(event == "PLAYER_REGEN_ENABLED") then
		inCombat = false;
	elseif(event == "PLAYER_REGEN_DISABLED") then
		inCombat = true;
	end

	-- Keep track of buff changes. Switch specs as required
	local myself = UnitGUID("player");		
	if(event == "COMBAT_LOG_EVENT_UNFILTERED") and (srcGUID == myself) and (dstGUID == myself)
	and ((eventType == "SPELL_AURA_APPLIED") or (eventType == "SPELL_AURA_REMOVED")) then
		AngeleDei:LoadSpec(false, true);		-- Don't warn of unsupported spec if all we're doing is changing stances
	end

	-- Don't do anything else if the current spec is unsupported!
	if(spec == nil) then
		return;
	end

	-- Handle the very first instance of either PLAYER_ALIVE or PLAYER_DEAD. These events indicate that
	-- all talent information is available to the UI. We'll use it to reload the Spellinfo object
	if(event == "PLAYER_ALIVE") or (event == "PLAYER_DEAD") then
		--print("[AngeleDei] |cFFFF0000" .. event .. "|r");
		AngeleDei:UnregisterEvent("PLAYER_ALIVE");	-- Do it only once!
		AngeleDei:UnregisterEvent("PLAYER_DEAD");
		AngeleDei:LoadSpec(true);
		return;
	end
	
	-- Process action bar page changes (duh)
	if(event == "ACTIONBAR_PAGE_CHANGED") then
		AngeleDei:RotationByActionBarPage();
		return
	end
	
	-- If the "combat toggle" option is set, enable the addon when we enter combat
	-- and disable it when combat ends.
	if(event == "PLAYER_REGEN_ENABLED") then
		if(config.combatToggle) then
			AngeleDei:DisableAddon();
		end
		return;
	elseif(event == "PLAYER_REGEN_DISABLED") then
		if(config.quietStart) then
			nextSound = GetTime() + 15;
		end
		if(config.combatToggle) then
			AngeleDei:EnableAddon();
		end
		return;
	end
	
	-- Process aura changes (used for vengeance)
	if(event == "UNIT_AURA" and config.vengeanceEnabled) then
		if(timestamp == "player") then
			AngeleDei:UpdateVengeanceIndicator();
		end
		return;
	end
	
	-- Re-run the simulation if the player changed the current target to
	-- or from a low-health target
	if(event == "PLAYER_TARGET_CHANGED") then
		-- Don't re-run the sim if we are clearing the target OR if the new target is not attackable
		if(not UnitExists("target")) or (not UnitCanAttack("player", "target")) or (UnitIsDead("target")) then
			return;
		end
	
		-- Copy all our debuffs from the target into state
		spec.state:CopyAllDebuffs(spec.spellinfo);
	
		-- Make sure we don't switch to the next GCD on very quick target changes
		-- If we don't subtract a GCD from GetTime(), we might skip a GCD completely, forcing a needlessly long wait
		-- before the next ability
		spec.state:AdjustTLast(GetTime() - spec.spellinfo:GCD(), SIM_SHIFT, spec.spellinfo);
		AngeleDei:RunSimulation(GetTime(), "target change", true);
		return;
	end
	
	-- Debug output for all combat events
	--if(srcGUID == myself) or (dstGUID == myself) then 
	--	if(eventType == "SPELL_MISSED") then
	--		ChatFrame1:AddMessage(myself .. ": |cFFFF0000" .. stringify(eventType) .. "|r/" .. stringify(p2) .. " (" .. stringify(srcName) .. ":" .. stringify(dstName) .. ") #1:" .. stringify(s1) .. " #2:" .. stringify(s2));
	--	end
	--end
	
	-- Ignore spellcasts and energy restore procs generated by other people
	if((eventType == "SPELL_CAST_SUCCESS") or (eventType == "SPELL_ENERGIZE")) and (srcGUID ~= myself) then
		return;
	end
	
	-- Ignore all other events that are not related to me
	if(srcGUID ~= myself) and (dstGUID ~= myself) then
		return;
	end
	
	-- Update the simulation based on combat log events
	if(spec.spellinfo:IsKnownSpell(p1)) then
		updateSimulation(
			event, GetTime(), eventType, hideCaster, 
			srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, 
			p1, p2, p3, s1, s2, s3, s4, s5, s6, s7);
	end
end

-- Process an event that could make us re-run the simulation
function updateSimulation(
	event, timestamp, eventType, hideCaster, 
	srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, 
	p1, p2, p3, s1, s2, s3, s4, s5, s6, s7)
	
	-- Update our state
	local needUpdate = spec.state:OnEvent(
		event, timestamp, eventType, hideCaster, 
		srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags,
		p1, p2, p3, s1, s2, s3, s4, s5, s6, s7, 
		spec.spellinfo);
	
	-- If the addon is disabled, we will not update the frame (but we will still
	-- maintain our state!)
	--if(not config.enabled) then
	--	return;
	--end
	
	-- If the rotation is not target-sensitive, do not re-run the sim when the target changes.
	-- This should reduce jitter for AOE rotations when target die quickly
	if(eventType == "PLAYER_TARGET_CHANGED") and (not Rotation:IsTargetSensitive()) then
		return;
	end

	local t = GetTime();
	
	if(needUpdate == SIM_SHIFT) then
		--DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFFAngele Dei|r: Shift (" .. eventType .. " / " .. p2 .. ")", 1, 1, 1);
		AngeleDei:Shift(GetTime());
	end

	if(needUpdate ~= SIM_IGNORE) then
		--DEFAULT_CHAT_FRAME:AddMessage("|cFF00FF80Angele Dei|r: Simulation (" .. eventType .. " / " .. p2 .. ")", 1, 1, 1);
		AngeleDei:RunSimulation(t, (needUpdate or "!") .. ":" .. (p2 or "?") .. "/" .. string.lower(eventType));
		--local list = Rotation:GetNext(5, GetTime(), Spellinfo, State);
		--DEFAULT_CHAT_FRAME:AddMessage("[|cFFFFC000Angeli Dei|r] Next " .. #list .. ":");			
		--for i=1,#list do
		--	if(lastList == nil) or (lastList[i+1] == nil) or (lastList[i+1] == list[i]) then		
		--		DEFAULT_CHAT_FRAME:AddMessage("    " .. list[i]);
		--	else
		--		DEFAULT_CHAT_FRAME:AddMessage("    " .. list[i] .. " (was: " .. lastList[i+1] .. ")");
		--	end
		--end
		--lastList = list;
	end
end

-- Re-run a simulation and modify the queue accordingly
function AngeleDei:RunSimulation(t, reason, suppressSound)
	--DEFAULT_CHAT_FRAME:AddMessage("[|cFFFFD000Angeli Dei|r] Next " .. (#queue-1) .. " -- |cFFFFD000" .. reason .. "|r @ " .. format("%4.3f", t) ..
	--	" | HP=|cFFFF8040" .. UnitPower("player", 9) .. "|r:|cFF8040FF" .. State:GetHolyPower() .. "|r");
	
	--DEFAULT_CHAT_FRAME:AddMessage("[|cFFFFD000AD|r] Sim(" .. (reason or "n/a") .. "): |cFFFFFF00" .. getHolyPower() .. "|r:|cFF00C0FF" .. spec.state:GetHolyPower() .. " |cFFC000FF @" .. GetTime() .. "|r");
	--DEFAULT_CHAT_FRAME:AddMessage("[|cFFFFD000AD|r] Sim(" .. (reason or "n/a") .. "): |cFFC000FF @" .. GetTime() .. "|r GCD:" .. spec.spellinfo:GCD());
	
	--DEFAULT_CHAT_FRAME:AddMessage("[|cFFFFD000AD|r] Sim(): Swipe CD: " .. (spec.state:GetCooldown(ABILITY_SWIPE) or "n/a"));
	
	local t0 = spec.state:GetEffectiveTLast() + (config.priority or 300)/1000.0; -- + spec.spellinfo:GCD();
	local list = spec:GetRotationByType(aoeMode):Run(t0, #queue, spec.state, spec.spellinfo);
	--DEFAULT_CHAT_FRAME:AddMessage("[|cFFFFD000AD|r] Sim(): " .. #list .. "; T-last: |cFFC000FF" .. spec.state:GetEffectiveTLast() .. " (|cFFFF0000" ..
	--	format("%6.5f", GetTime() - spec.state:GetEffectiveTLast()) .. "|r)");

	local changed = nil;
	for i=1,#list-1 do
		--DEFAULT_CHAT_FRAME:AddMessage("|cFF00FFFF    {" .. i .. "} " .. (GetSpellInfo(list[i].ability) or "?") .. " @ |cFF40FFFF" .. format("%4.3f", list[i].time) .. "|r / " .. (list[i].idle and "idle" or "-"));

		local x = queue[i];
		local a = list[i];
		x.previous = x.ability;
		x.ability = a.ability;
		x.time = a.time;
		x.idle = a.idle;
		x:SetTexture(spec.spellinfo:GetIcon(x.ability));
		x.label:SetText(keystrokes[x.ability]);

		if(onEnable) then
			x.previous = x.ability;
			x.blinkStart = nil;
			x.blinkEnd = nil;
		else
			-- Blink it if it has changed
			if(x.previous ~= x.ability) then
				-- We'll use it to choose the sound
				if(changed == nil) then
					changed = i;
				end
				x.blinkStart = t;
				x.blinkEnd = t+0.45;			
			end
		end
		
	end

	if(changed) and (changed == 1) and (not suppressSound) then
		AngeleDei:PlayRotationChangeWarning();
	end
end

-- Play a warning sound file when the rotation changes
function AngeleDei:PlayRotationChangeWarning()
	if(not config.sound) then
		return;
	end
	
	local t = GetTime();
	
	if(t < nextSound) then
		return;
	end
	
	-- Make sure we don't play any sounds when we initiate combat by using an ability
	-- (this only affects abilities with a delay between cast time and effect time,
	-- for example AS or J)
	if(not UnitAffectingCombat("player")) and (config.quietStart) then
		return;
	end
	
	local file = sound[1];
	nextSound = t + 1;

	if(config.reduceVolume) then
		file = sound[soundVolume];
		soundVolume = soundVolume+1;
		if(soundVolume > #sound) then
			soundVolume = 1;
			nextSound = t + 5.5;
		end
	end

	PlaySoundFile("Interface\\AddOns\\AngeleDei\\sounds\\" .. file, config.masterChannel and "Master" or nil);
end

-- Create a new, empty texture. We'll need five of them
function AngeleDei:createEmptyTexture(x, alpha, ability, text)
	local t = AngeleDei_RotationFrame:CreateTexture();
	t.ability = ability;
	
	local label = AngeleDei_RotationFrame:CreateFontString();
	label:SetPoint("TOPRIGHT", t, "TOPRIGHT", 0, -2);
	label:SetFont(FONT, 12, "OUTLINE");
	label:SetTextColor(1, 1, 1, 1);
	label:SetShadowColor(0, 0, 0, 0.5);
	label:SetShadowOffset(1.5, -1.5);
	label:SetText(text or "A");	
	t.label = label;
	
	-- These are related to blink notifications when the icon changes
	t.previous = nil;
	t.blinkStart = nil;
	t.blinkEnd = nil;

	t.SetAlpha_Old = t.SetAlpha;
	t.SetAlpha = function(obj, x)		-- Hmm. This is kinda neat...
		obj:SetAlpha_Old(x);
		obj.label:SetAlpha(x);
	end
	
	t.SetWidth_Old = t.SetWidth;
	t.SetWidth = function(obj, x)
		obj:SetWidth_Old(x);
		obj.label:SetFont(FONT, 12*x/32, "OUTLINE");
	end	

	t:SetPoint("CENTER", AngeleDei_RotationFrame, "LEFT", x, 0);
	--t:SetTexture("Interface\\Icons\\Spell_Holy_CrusaderStrike");
	t:SetWidth(32);
	t:SetHeight(32);
	t:SetAlpha(alpha or 1.0);
	t:Show();

	return t;
end

-- Switch to a single target or AOE rotation depending on the argument
function AngeleDei:SwitchToRotation(aoe, suppressVisuals)
	aoeMode = aoe;
	keystrokes = spec:GetKeystrokes(aoeMode);

	spec.state:SetHolyPower(UnitPower("player", 9));
	spec.state:AdjustTLast(GetTime() - spec.spellinfo:GCD(), SIM_SHIFT, spec.spellinfo);
	AngeleDei:RunSimulation(GetTime(), "On rotation change", true);
	if(not suppressVisuals) then
		AngeleDei:ShowRotationChangeMessage();
	end
end

-- Show the rotation change message
function AngeleDei:ShowRotationChangeMessage()
	if(not config.showRotation) then
		return;
	end
	
	local mess = AngeleDei_RotationFrame.message;
	if(mess == nil) then
		mess = AngeleDei_RotationFrame:CreateFontString(nil, "ARTWORK");
		mess:SetFont(MESSAGE_FONT, 36, "THICKOUTLINE");
		mess:SetTextColor(1, 0.9, 0.9, 1);
		--mess:SetShadowColor(1, 0, 0, 0.5);
		--mess:SetShadowOffset(5, -5);
		AngeleDei_RotationFrame.message = mess;
	end
	
	mess:SetText(aoeMode and "AoE" or "Single target");
	mess:SetPoint("CENTER", UIParent, "CENTER");
	mess:SetAlphaGradient(0, 4);
	mess:Show();
	
	messageStart = GetTime();
	messageShown = messageStart + 0.25;
	messageFadeOutStart = messageShown + 1.5;
	messageEnd = messageFadeOutStart + 0.25;
end

-- Switch between single-target and AOE rotations
function AngeleDei:ToggleRotation()
	AngeleDei:SwitchToRotation(not aoeMode);
end

-- Switch to the rotation that matches the given action bar
function AngeleDei:RotationByActionBarPage(suppressVisuals)
	local page = GetActionBarPage();
	
	local newMode = aoeMode;
	if(page == config.singlePage) then
		newMode = false;
	elseif(page == config.aoePage) then
		newMode = true;
	end
	
	if(newMode == aoeMode) then
		return;
	end
	
	-- The mode has changed
	self:SwitchToRotation(newMode, suppressVisuals);
end

-- Extract tooltip text from a list of tooltip regions
local function getTooltipText(...)
	local text = "";
	for i=1,select("#", ...) do
		local rgn = select(i, ...);
		if(rgn) and (rgn:GetObjectType() == "FontString") then
			text = text .. (rgn:GetText() or "");
		end
	end
	return text == "" and "0" or text;
end

-- Get the current Vengeance (0.0 .. 1.0)
function AngeleDei:GetVengeance()
	local name = GetSpellInfo(VENG);
	local n = UnitAura("player", name);
	if(n == nil) then
		return 0;
	end
	
	local s = AngeleDei_RotationFrame.scanner;
	s:ClearLines();
	s:SetUnitBuff("player", name);
	local text = getTooltipText(s:GetRegions());
	local vengeance = tonumber(string.match(text, "%d+"));
	local maxVengeance = floor(0.1*UnitHealthMax("player"));
	local result = vengeance / maxVengeance;
	return result;
end

-- Update the vengeance indicator
function AngeleDei:UpdateVengeanceIndicator()
	local v = AngeleDei:GetVengeance();
	--print("Vengeance: " .. v);
	
	for i=0,4 do
		local t = AngeleDei_RotationFrame.vengeanceLight[i];
		local v0 = i*0.2;
		local v1 = (i+1)*0.2;
		if(v <= v0) then
			t:SetAlpha(0);
		elseif(v > v1) then
			t:SetAlpha(1);
		else
			t:SetAlpha((v-v0)/0.2);
		end			
	end
end