-- Feral (bear) spec for Druids
BearSpec = new(Spec);

local DEFAULT_SINGLE_TARGET = "Default single target";
local DEFAULT_HT_SINGLE_TARGET = "Default high-threat single target";
local DEFAULT_AOE = "Default AoE";

-- Construct the default single-target rotation
local function createSingleTargetRotation(self, name)
	local r = Rotation:new();
	
	local re = RotationElement:new(ABILITY_DEMOROAR);
	re.buffs = { [DEBUFF_DEMOROAR]=false };
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_MANGLE);
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_FAERIE_FIRE);
	re.stackMax = { [DEBUFF_FAERIE_FIRE]=2 };
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_PULVERIZE);
	re.buffs = { [EFFECT_PULVERIZE]=false };
	re.stackMin = { [DEBUFF_LACERATE]=3 };
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_LACERATE);
	r:add(re);
	
	r:SetName(name);
	r.locked = true;	
	self.rotations[r:GetName()] = r;
end

-- Construct a high-threat single-target rotation
local function createHighThreatSingleTargetRotation(self, name)
	local r = Rotation:new();
	
	local re = RotationElement:new(ABILITY_DEMOROAR);
	re.buffs = { [DEBUFF_DEMOROAR]=false };
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_MANGLE);
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_THRASH);
	re.stackMin = { [DEBUFF_LACERATE]=3 };
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_FAERIE_FIRE);
	re.stackMax = { [DEBUFF_FAERIE_FIRE]=2 };
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_PULVERIZE);
	re.buffs = { [EFFECT_PULVERIZE]=false };
	re.stackMin = { [DEBUFF_LACERATE]=3 };
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_LACERATE);
	r:add(re);
	
	r:SetName(name);
	r.locked = true;	
	self.rotations[r:GetName()] = r;
end

-- Construct the default AoE rotation
local function createAoERotation(self, name)
	local r = Rotation:new();	
	
	local re = RotationElement:new(ABILITY_DEMOROAR);
	re.buffs = { [DEBUFF_DEMOROAR]=false };
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_MANGLE);
	re.buffs = { [EFFECT_BERSERK]=true };
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_SWIPE);
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_THRASH);
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_MANGLE);
	r:add(re);

	r:SetName(name);
	r.locked = true;	
	self.rotations[r:GetName()] = r;
end

-- Initialize the instance
function BearSpec:Init()
	self.state = new(BearState);
	self.spellinfo = new(BearSpellinfo);
	
	self.singleRotation = self:GetDefaultSingleRotation();
	self.aoeRotation = self:GetDefaultAOERotation();
	
	-- Create default rotations
	createSingleTargetRotation(self, DEFAULT_SINGLE_TARGET);
	createHighThreatSingleTargetRotation(self, DEFAULT_HT_SINGLE_TARGET);
	createAoERotation(self, DEFAULT_AOE);
end

-- Get the name of the class supported by this spec
function BearSpec:GetClass()
	return "DRUID";
end

-- Get the ID of the talent tree supported by this spec
function BearSpec:GetTalentTree()
	return 2;		-- Feral
end

-- Get the description of this spec
function BearSpec:GetDescription()
	return "Druid (Feral: Bear)", "|cFFFF8000Druid|r (Feral: Bear)";
end

-- Get the label of the cost range input boxes
function BearSpec:GetCostRangeLabel()
	return "Rage";
end

-- Get name of the default single-target rotation
function BearSpec:GetDefaultSingleRotation()
	return DEFAULT_SINGLE_TARGET;
end

-- Get name of the default AOE rotation
function BearSpec:GetDefaultAOERotation()
	return DEFAULT_AOE;
end

-- Get the buff indicating the required stance
function BearSpec:GetRequiredStanceBuff()
	return 5487;		-- Bear form
end

-- Add this spec to the list of supported specs
SpecList:Add(new(BearSpec));
