BearState = new(State);

local max = math.max;
local last = nil;

-- Update the state using the data of a combat log event.
-- Return SIM_IGNORE if the event was ignorable, SIM_SHIFT if the icons should be shifted and rotation recalculated,
-- or SIM_RECALCULATE if the rotation should be recalculated without shifting the icons
function BearState:OnEventInternal(
	event, timestamp, eventType, hideCaster, 
	srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, 
	p1, p2, p3, s1, s2, s3, s4, s5, s6, s7, 
	spellinfo)

	local t = GetTime();
	local spellID = p1;
	local myself = UnitGUID("player");
	local target = UnitGUID("target");

	--[[
	
	--if(eventType == "SPELL_CAST_SUCCESS") then
	--	local dt = "";
	--	if(last ~= nil) then
	--		print("[|cFF90FF80" .. GetSpellInfo(spellID) .. "|r]: |cFFC0C0FF" .. format("%4.3f", t-last));
	--	end
	--	last = t;		
	--end

	local dt = "";
	if(last ~= nil) then
		dt = " |r(|cFFC0C0FF" .. format("%4.3f", t-last) .. "|r)";
	end

	local sn = srcName;
	local dn = dstName;
	
	if(srcGUID == myself) then
		sn = "|cFF00FF00me|r";
	end
	if(dstGUID == myself) then
		dn = "|cFF00FF00me|r";
	end
	
	if(srcGUID == target) then
		sn = "|cFF008000T|r";
	end
	if(dstGUID == target) then
		dn = "|cFF008000T|r";
	end

	--if(eventType == "SPELL_DAMAGE") and (spellID == ABILITY_MAUL) then
	--	print("[|cFFFFFF00AD|r] " .. string.lower(eventType) .. ", |cFFC0FF00" .. p2 .. "|r:" .. p1 .. " - " .. s1 .. (s7 and "|cFFFF0000!|r" or "")
	--		.. " @ |cFFE080FF" .. GetTime() .. dt);
	--	last = t;
	--end

	if(eventType ~= "SPELL_PERIODIC_DAMAGE") then
		print("[|cFFFFFF00AD|r] " .. string.lower(eventType) .. ", " .. (sn or "?") .. ":" .. (dn or "?") .. ", |cFFC0FF00" .. p2 .. "|r:" .. p1
			.. " @ |cFFE080FF" .. GetTime() .. dt);
		last = t;
	end
	
	--]]

	local result = SIM_IGNORE;

	-- This message is not ours	
	if(srcGUID ~= myself) and (dstGUID ~= myself) then
		return result;
	end
	
	-- Berserk proc resets Mangle CD
	-- Using Berserk cooldown should recalculate the entire rotation (a.k.a. zerkspam)
	if(eventType == "SPELL_AURA_APPLIED") and ((spellID == EFFECT_BERSERK_PROC) or (spellID == EFFECT_BERSERK)) then
		self:SetCooldown(ABILITY_MANGLE, nil);
		result = max(SIM_RECALCULATE, result);
	end
	
	-- Misses on the target will require a re-sync of debuffs. Misses on other mobs will be ignored
	if(eventType == "SPELL_MISSED") and (spellinfo:IsKnownSpell(spellID)) and (dstGUID == target) then
		self:CopyAllDebuffs(spellinfo);
		result = max(SIM_RECALCULATE, result);
		
		-- In case of Pulverize, CopyAllDebuffs has already restored the stack of Lacerate
		-- we have removed, but we still need to re-sync the Pulverize buff...
		local expiration = adBuffExpiration(EFFECT_PULVERIZE);
		self:SetExpiration(EFFECT_PULVERIZE, expiration);
	end

	-- Spells
	if(eventType == "SPELL_CAST_SUCCESS") then
		if(spellinfo:IsKnownAbility(spellID)) then
			self:SetCooldown(spellID, t+spellinfo:GetCooldown(spellID));
			if(spellID == ABILITY_MAUL) then
				result = max(SIM_RECALCULATE, result);		-- Mangle is off GCD
			else
				result = max(SIM_SHIFT, result);			-- Everything else is on GCD
			end
		end

		-- Mangle has no cooldown while Berserk is active
		if(spellID == ABILITY_MANGLE) and (self:IsActive(EFFECT_BERSERK, t)) then
			self:SetCooldown(spellID, nil);
			self:SetExpiration(EFFECT_BERSERK_PROC, nil);
			result = max(SIM_SHIFT, result);
		end
		
		-- Demoralizing Roar applies a debuff
		if(spellID == ABILITY_DEMOROAR) then
			self:ApplyDebuff(DEBUFF_DEMOROAR, t + spellinfo:GetExpiration(DEBUFF_DEMOROAR), t, spellinfo);
			result = max(SIM_SHIFT, result);
		end
		
		-- Lacerate applies a debuff
		if(spellID == ABILITY_LACERATE) then
			self:ApplyDebuff(DEBUFF_LACERATE, t + spellinfo:GetExpiration(DEBUFF_LACERATE), t, spellinfo);
			result = max(SIM_SHIFT, result);
		end
			
		-- Mangle applies two debuffs
		if(spellID == ABILITY_MANGLE) then
			self:ApplyDebuff(DEBUFF_INFECTED, t + spellinfo:GetExpiration(DEBUFF_INFECTED), t, spellinfo);
			self:ApplyDebuff(DEBUFF_MANGLE, t + spellinfo:GetExpiration(DEBUFF_MANGLE), t, spellinfo);
			result = max(SIM_SHIFT, result);
		end
		
		-- Thrash applies a debuff
		if(spellID == ABILITY_THRASH) then
			self:ApplyDebuff(DEBUFF_THRASH, t + spellinfo:GetExpiration(DEBUFF_THRASH), t, spellinfo);
			result = max(SIM_SHIFT, result);
		end

		-- Maul applies a debuff
		if(spellID == ABILITY_MAUL) then
			self:ApplyDebuff(DEBUFF_INFECTED, t + spellinfo:GetExpiration(DEBUFF_INFECTED), t, spellinfo);
			result = max(SIM_RECALCULATE, result);
		end
		
		-- Pulverize applies a buff and consumes stacks of Lacerate
		if(spellID == ABILITY_PULVERIZE) then
			self:SetExpiration(EFFECT_PULVERIZE, t + spellinfo:GetExpiration(EFFECT_PULVERIZE));
			self:RemoveDebuff(DEBUFF_LACERATE);
			result = max(SIM_RECALCULATE, result);
		end
		
		-- Faerie fire applies a debuff depending on the rank of Feral Aggression
		if(spellID == ABILITY_FAERIE_FIRE) then
			local stacks = talentRank(16859)+1;
			for i=1,stacks do
				self:ApplyDebuff(DEBUFF_FAERIE_FIRE, t + spellinfo:GetExpiration(DEBUFF_FAERIE_FIRE), t, spellinfo);
			end
			result = max(SIM_RECALCULATE, result);
		end

		result = max(SIM_IGNORE, result);
	end
	
	-- Debuffs
	if(srcGUID == myself) and (dstGUID == target) and
	((eventType == "SPELL_AURA_APPLIED") or (eventType == "SPELL_AURA_APPLIED_DOSE") or (eventType == "SPELL_AURA_REFRESH") or (eventType == "SPELL_AURA_REMOVED")) then
		if(spellinfo:IsKnownSpell(spellID)) then
			self:CopyDebuff(spellID);
			result = max(SIM_RECALCULATE, result);
		end
		result = max(SIM_IGNORE, result);
	end
	
	-- Buffs
	if(srcGUID == myself) and (dstGUID == myself) and (spellinfo:IsKnownSpell(spellID)) then
		if(eventType == "SPELL_AURA_APPLIED") or (eventType == "SPELL_AURA_APPLIED_DOSE") or (eventType == "SPELL_AURA_REFRESH") then
			local expiration = adBuffExpiration(spellID);
			self:SetExpiration(spellID, expiration);
			
			-- For some reason, the Pulverize buff is applied before cast success is reported.
			-- This causes an unwanted warning, so we'll ignore it.
			if(spellID == EFFECT_PULVERIZE) then
				result = max(SIM_IGNORE, result);
			else
				result = max(SIM_RECALCULATE, result);
			end
		end
		
		result = max(SIM_IGNORE, result);
	end

	-- Lacerate debuff doesn't receive applied or refresh events after it reaches 3 stacks,
	-- so we'll use initial damage instead
	if(eventType == "SPELL_DAMAGE") and (spellID == ABILITY_LACERATE) then
		local _, stack = self:GetDebuff(spellID, t);
		if(stack == 3) then
			self:CopyDebuff(spellID);
			result = max(SIM_RECALCULATE, result);
		end
	end

	return max(SIM_IGNORE, result);
end

-- Apply the given ability to the state object
function BearState:ApplyAbility(id, t, spellinfo)
	if(id == IDLE) then
		return;
	end
	
	local target = UnitGUID("target") or -1;
	
	-- Lacerate: apply a dose of the debuff
	if(id == ABILITY_LACERATE) then
		self:ApplyDebuff(DEBUFF_LACERATE, t + spellinfo:GetExpiration(DEBUFF_LACERATE), t, spellinfo);
		return;
	end
	
	-- Pulverize: apply the buff, remove Lacerate from target
	if(id == ABILITY_PULVERIZE) then
		self:SetExpiration(EFFECT_PULVERIZE, t + spellinfo:GetExpiration(EFFECT_PULVERIZE));
		self:RemoveDebuff(DEBUFF_LACERATE);
		return;
	end
	
	-- Mangle: start the cooldown, apply IW and Mangle debuffs, remove Berserk buff
	if(id == ABILITY_MANGLE) then
		self:ApplyDebuff(DEBUFF_INFECTED, t + spellinfo:GetExpiration(DEBUFF_INFECTED), t, spellinfo);
		self:ApplyDebuff(DEBUFF_MANGLE, t + spellinfo:GetExpiration(DEBUFF_MANGLE), t, spellinfo);
		self:SetExpiration(EFFECT_BERSERK_PROC, nil);
		if(self:IsActive(EFFECT_BERSERK, t)) then
			self:SetCooldown(id, nil);								-- No cooldown on Mangle under Berserk
		else
			self:SetCooldown(id, t + spellinfo:GetCooldown(id));
		end
		return;
	end
	
	-- Demo roar: apply the debuff
	if(id == ABILITY_DEMOROAR) then
		self:ApplyDebuff(DEBUFF_DEMOROAR, t + spellinfo:GetExpiration(DEBUFF_DEMOROAR), t, spellinfo);
		return;
	end
	
	-- Maul: start the cooldown, apply IW
	if(id == ABILITY_MAUL) then
		self:SetCooldown(id, t + spellinfo:GetCooldown(id));
		self:ApplyDebuff(DEBUFF_INFECTED, t + spellinfo:GetExpiration(DEBUFF_INFECTED), t, spellinfo);
		return;
	end
	
	-- Thrash: start the cooldown, apply the debuff
	if(id == ABILITY_THRASH) then
		self:SetCooldown(id, t + spellinfo:GetCooldown(id));
		self:ApplyDebuff(DEBUFF_THRASH, t + spellinfo:GetExpiration(DEBUFF_THRASH), t, spellinfo);
		return;
	end
	
	-- Swipe: start the cooldown
	if(id == ABILITY_SWIPE) then
		self:SetCooldown(id, t + spellinfo:GetCooldown(id));
		return;
	end
	
	-- Faerie Fire: start the cooldown and apply the required number of debuffs
	if(id == ABILITY_FAERIE_FIRE) then
		local stacks = talentRank(16859)+1;
		for i=1,stacks do
			self:ApplyDebuff(DEBUFF_FAERIE_FIRE, t + spellinfo:GetExpiration(DEBUFF_FAERIE_FIRE), t, spellinfo);
		end
		return;
	end
		
	-- Huh...
	self:SetCooldown(id, t + spellinfo:GetCooldown(id));
end
