--
-- Parameters:
--
--		id			- ability spell ID
--		count		- ability position in the rotation (usually "X of Y")
--		data		- the table whose contents we are editing
--		saveHandler	- the function invoked when the user clicks "Save"; the argument is self.data
--		effects		- spell IDs of checked buffs or debuffs (sorted)

Conditions = CreateFrame("Frame", "AngeleDeiConditions", UIParent);

local MAX_EFFECTS = 10;
local EMPTY = { };
local FONT = "Fonts\\ARIALN.TTF";	--"Interface\\AddOns\\AngeleDei\\fonts\\russel square lt.ttf";

-- Create the Conditions frame but don't populate it
function Conditions:Construct()
	local backdrop = {
		--bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background", 
		--edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
		--bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", 
		--edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		--bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", 
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		tile = true,
		tileSize = 24,
		edgeSize = 24,
		insets = {
			left = 8,
			right = 8,
			top = 8,
			bottom = 8
		}
	};
	
	self:EnableMouse(true);
	self:SetBackdrop(backdrop);
	self:SetBackdropColor(0, 0, 0, 1);
	self:SetFrameLevel(80);

	self:SetWidth(400);
	self:SetHeight(388);
	self:SetPoint("CENTER", Sequence, "CENTER", 0, 0);
	
	self.icon = self:CreateTexture(self:GetName() .. "Icon");
	self.icon:SetWidth(48);
	self.icon:SetHeight(48);
	self.icon:SetTexture("Interface\\PaperDoll\\Ui-paperdoll-slot-secondaryhand"); 
	self.icon:SetPoint("TOPLEFT", self, "TOPLEFT", 16, -16);
	
	self.nameText = self:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	self.nameText:SetPoint("BOTTOMLEFT", self.icon, "RIGHT", 8, 4);
	self.nameText:SetText("Ability name goes here");

	self.countText = self:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	self.countText:SetPoint("TOPLEFT", self.icon, "RIGHT", 8, -4);
	self.countText:SetText("2 of 6");
	self.countText:SetVertexColor(1, 1, 1);
	
	self.icons = { };
	self.borders = { };
	self.active = { };
	self.labels = { };
	self.minTime = { };
	self.maxTime = { };
	self.minStack = { };
	self.maxStack = { };
	
	for i=1,MAX_EFFECTS do
		local icon = self:CreateTexture(self:GetName() .. "Icon" .. i);
		icon:SetWidth(24);
		icon:SetHeight(24);
		icon:SetTexture("Interface\\PaperDoll\\Ui-paperdoll-slot-secondaryhand"); 
		icon:SetPoint("TOP", self.icon, "BOTTOMLEFT", 64 + 24/2 + 8 + (i-1)*(24+8), -16);
		icon:SetDrawLayer("ARTWORK", 1);
		self.icons[i] = icon;
		
		local border = self:CreateTexture(self:GetName() .. "Border" .. i);
		border:SetWidth(26);
		border:SetHeight(26);
		border:SetTexture("Interface\\AddOns\\AngeleDei\\images\\thick-outline.tga"); 
		border:SetPoint("CENTER", icon, "CENTER");
		border:SetAlpha(0.5);
		border:SetDrawLayer("ARTWORK", 2);
		self.borders[i] = border;
		
		local label = self:CreateFontString();
		label:SetPoint("BOTTOMRIGHT", icon, "BOTTOMRIGHT", 0, 2);
		label:SetFont(FONT, 10, "OUTLINE");
		label:SetTextColor(1, 1, 1, 1);
		--label:SetShadowColor(0, 0, 0, 0.5);
		--label:SetShadowOffset(1, -1);
		label:SetText("");
		label:SetDrawLayer("ARTWORK", 3);
		self.labels[i] = label;
	end

	local activeLabel = self:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	activeLabel:SetWordWrap(true);
	activeLabel:SetText("Must be active");
	activeLabel:SetWidth(64);
	activeLabel:SetJustifyH("LEFT");
	activeLabel:SetPoint("TOPLEFT", self.icon, "BOTTOMLEFT", 0, -44);	-- (-16-24-4)

	local minTimeLabel = self:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	minTimeLabel:SetWordWrap(true);
	minTimeLabel:SetText("Min. time remaining");
	minTimeLabel:SetWidth(64);
	minTimeLabel:SetJustifyH("LEFT");
	minTimeLabel:SetPoint("TOPLEFT", activeLabel, "TOPLEFT", 0, -32);

	local maxTimeLabel = self:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	maxTimeLabel:SetWordWrap(true);
	maxTimeLabel:SetText("Max. time remaining");
	maxTimeLabel:SetWidth(64);
	maxTimeLabel:SetJustifyH("LEFT");
	maxTimeLabel:SetPoint("TOPLEFT", minTimeLabel, "TOPLEFT", 0, -32);
	
	local minStackLabel = self:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	minStackLabel:SetWordWrap(true);
	minStackLabel:SetText("Min. stack");
	minStackLabel:SetWidth(64);
	minStackLabel:SetJustifyH("LEFT");
	minStackLabel:SetPoint("TOPLEFT", maxTimeLabel, "TOPLEFT", 0, -32);

	local maxStackLabel = self:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	maxStackLabel:SetWordWrap(true);
	maxStackLabel:SetText("Max. stack");
	maxStackLabel:SetWidth(64);
	maxStackLabel:SetJustifyH("LEFT");
	maxStackLabel:SetPoint("TOPLEFT", minStackLabel, "TOPLEFT", 0, -32);

	for i=1,MAX_EFFECTS do
		local c = self:CreateSwitch(self:GetName() .. "Active" .. i, self);
		c:SetPoint("TOP", self.icons[i], "BOTTOM", 0, -6);
		self.active[i] = c;

		c = CreateFrame("EditBox", self:GetName() .. "MinTime" .. i, self, "InputBoxTemplate");
		c:SetWidth(24);
		c:SetHeight(20);
		c:SetPoint("TOP", self.icons[i], "BOTTOM", 0, -32);
		c:SetJustifyH("LEFT");
		c:SetMultiLine(false);
		c:SetAutoFocus(false);
		c:SetScript("OnShow", function(this) this:SetCursorPosition(0); end);
		c:SetFont("Fonts\\ARIALN.TTF", 11, "");
		c:SetMaxLetters(4);
		self.minTime[i] = c;

		c = CreateFrame("EditBox", self:GetName() .. "MaxTime" .. i, self, "InputBoxTemplate");
		c:SetWidth(24);
		c:SetHeight(20);
		c:SetPoint("TOP", self.icons[i], "BOTTOM", 0, -32*2);
		c:SetJustifyH("LEFT");
		c:SetMultiLine(false);
		c:SetAutoFocus(false);
		c:SetScript("OnShow", function(this) this:SetCursorPosition(0); end);
		c:SetFont("Fonts\\ARIALN.TTF", 11, "");
		c:SetMaxLetters(4);
		self.maxTime[i] = c;

		c = CreateFrame("EditBox", self:GetName() .. "MinStack" .. i, self, "InputBoxTemplate");
		c:SetWidth(24);
		c:SetHeight(20);
		c:SetPoint("TOP", self.icons[i], "BOTTOM", 0, -32*3);
		c:SetJustifyH("LEFT");
		c:SetMultiLine(false);
		c:SetAutoFocus(false);
		c:SetScript("OnShow", function(this) this:SetCursorPosition(0); end);
		c:SetFont("Fonts\\ARIALN.TTF", 11, "");
		c:SetMaxLetters(4);
		self.minStack[i] = c;

		c = CreateFrame("EditBox", self:GetName() .. "MaxStack" .. i, self, "InputBoxTemplate");
		c:SetWidth(24);
		c:SetHeight(20);
		c:SetPoint("TOP", self.icons[i], "BOTTOM", 0, -32*4);
		c:SetJustifyH("LEFT");
		c:SetMultiLine(false);
		c:SetAutoFocus(false);
		c:SetScript("OnShow", function(this) this:SetCursorPosition(0); end);
		c:SetFont("Fonts\\ARIALN.TTF", 11, "");
		c:SetMaxLetters(4);
		self.maxStack[i] = c;
	end

	self.manaLabel = self:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	self.manaLabel:SetWordWrap(true);
	self.manaLabel:SetText("???");
	self.manaLabel:SetWidth(64);
	self.manaLabel:SetJustifyH("LEFT");
	self.manaLabel:SetPoint("TOPLEFT", maxStackLabel, "TOPLEFT", 0, -40);
	
	self.manaMin = CreateFrame("EditBox", "MinManaEditBox", self, "InputBoxTemplate");
	self.manaMin:SetWidth(48);
	self.manaMin:SetHeight(20);
	self.manaMin:SetPoint("LEFT", self.manaLabel, "RIGHT", 8, 0);
	self.manaMin:SetJustifyH("LEFT");
	self.manaMin:SetNumeric(true);
	self.manaMin:SetMultiLine(false);
	self.manaMin:SetAutoFocus(false);
	self.manaMin:SetScript("OnShow", function(this) this:SetCursorPosition(0); end);
	self.manaMin:SetFont("Fonts\\ARIALN.TTF", 11, "");
	self.manaMin:SetMaxLetters(4);

	local manaRangeLabel = self:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	manaRangeLabel:SetText("...");
	manaRangeLabel:SetWidth(12);
	manaRangeLabel:SetJustifyH("CENTER");
	manaRangeLabel:SetPoint("LEFT", self.manaMin, "RIGHT", 8, 0);
	
	self.manaMax = CreateFrame("EditBox", "MaxManaEditBox", self, "InputBoxTemplate");
	self.manaMax:SetWidth(48);
	self.manaMax:SetHeight(20);
	self.manaMax:SetPoint("LEFT", manaRangeLabel, "RIGHT", 8, 0);
	self.manaMax:SetJustifyH("LEFT");
	self.manaMax:SetNumeric(true);
	self.manaMax:SetMultiLine(false);
	self.manaMax:SetAutoFocus(false);
	self.manaMax:SetScript("OnShow", function(this) this:SetCursorPosition(0); end);
	self.manaMax:SetFont("Fonts\\ARIALN.TTF", 11, "");
	self.manaMax:SetMaxLetters(4);

	local hopoLabel = self:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	hopoLabel:SetWordWrap(true);
	hopoLabel:SetText("Min. holy power");
	hopoLabel:SetWidth(64);
	hopoLabel:SetJustifyH("LEFT");
	hopoLabel:SetPoint("TOPLEFT", self.manaLabel, "TOPLEFT", 0, -32);

	self.hopo = CreateFrame("EditBox", "MinHoPoEditBox", self, "InputBoxTemplate");
	self.hopo:SetWidth(48);
	self.hopo:SetHeight(20);
	self.hopo:SetPoint("LEFT", hopoLabel, "RIGHT", 8, 0);
	self.hopo:SetJustifyH("LEFT");
	self.hopo:SetNumeric(true);
	self.hopo:SetMultiLine(false);
	self.hopo:SetAutoFocus(false);
	self.hopo:SetScript("OnShow", function(this) this:SetCursorPosition(0); end);
	self.hopo:SetFont("Fonts\\ARIALN.TTF", 11, "");
	self.hopo:SetMaxLetters(1);

	self.saveButton = CreateFrame("Button", self:GetName() .. "ConditionsOK", self, "UIPanelButtonTemplate");
	self.saveButton:SetText("Save");
	self.saveButton:SetWidth(64);
	self.saveButton:SetHeight(24);
	self.saveButton:SetPoint("TOPLEFT", hopoLabel, "BOTTOMLEFT", 0, -20);
	self.saveButton:SetScript("OnClick", function()
		-- Save the contents
		local updated = self:Save();
		self:Hide();
		if(self.saveHandler ~= nil) then
			self.saveHandler(updated);
		end
	end);

	self.cancelButton = CreateFrame("Button", self:GetName() .. "ConditionsCancel", self, "UIPanelButtonTemplate");
	self.cancelButton:SetText("Cancel");
	self.cancelButton:SetWidth(64);
	self.cancelButton:SetHeight(24);
	self.cancelButton:SetPoint("TOPLEFT", self.saveButton, "TOPRIGHT", 6, 0);
	self.cancelButton:SetScript("OnClick", function()
		self:Hide();
		if(self.cancelHandler ~= nil) then
			self.cancelHandler();
		end
	end);
	
	self:Hide();
end

-- Save the contents of the dialog to the 'data' field
function Conditions:Save()
	local data = { };
	
	data.id = self.data.id;
	
	-- Save effect-keyed properties
	for i=1,#self.effects do
		local id = self.effects[i];
		
		local state = self.active[i]:GetValue();
		if(state == "yes") or (state == "no") then
			data.buffs = data.buffs or { };
			data.buffs[id] = (state == "yes") and true or false;
		end
		
		local minT = self.minTime[i]:GetText();
		if(minT ~= nil) and (string.trim(minT) ~= "") then
			data.remainingMin = data.remainingMin or { };
			data.remainingMin[id] = 0 + minT;		-- Force a string-to-number conversion
		end

		local maxT = self.maxTime[i]:GetText();
		if(maxT ~= nil) and (string.trim(maxT) ~= "") then
			data.remainingMax = data.remainingMax or { };
			data.remainingMax[id] = 0 + maxT;
		end

		local minS = self.minStack[i]:GetText();
		if(minS ~= nil) and (string.trim(minS) ~= "") then
			data.stackMin = data.stackMin or { };
			data.stackMin[id] = 0 + minS;
		end

		local maxS = self.maxStack[i]:GetText();
		if(maxS ~= nil) and (string.trim(maxS) ~= "") then
			data.stackMax = data.stackMax or { };
			data.stackMax[id] = 0 + maxS;
		end
		
		--print("[" .. i .. "] min:<" .. (minT or "") .. "> max:<" .. (maxT or "") .. ">");
	end
	
	-- Save other properties
	local hopo = self.hopo:GetText();
	if(hopo ~= nil) and (string.trim(hopo) ~= "") then
		data.hopo = 0 + hopo;
	end

	local manaMin = self.manaMin:GetText();
	if(manaMin ~= nil) and (string.trim(manaMin) ~= "") then
		data.manaMin = 0 + manaMin;
	end

	local manaMax = self.manaMax:GetText();
	if(manaMax ~= nil) and (string.trim(manaMax) ~= "") then
		data.manaMax = 0 + manaMax;
	end

	return data;
end

-- Create a yes-no-ignore button
function Conditions:CreateSwitch(name, parent)
	local f = CreateFrame("Frame", name, parent);
	f:SetWidth(32);
	f:SetHeight(16);
	f.state = nil;
	
	local s = f:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	s:SetAllPoints(f);
	s:SetJustifyH("CENTER");
	s:SetText("|cFFFFC000-");
	f.text = s;
	
	f:EnableMouse(true);
	f:SetScript("OnMouseDown", function()
		local s;
		if(f.state == "yes") then
			s = "no";
		elseif(f.state == "no") then
			s = nil;
		else
			s = "yes";
		end
		f:SetValue(s);
	end);

	f.SetValue = function(this, n)
		f.state = n;
		if(f.state == "yes") then
			f.text:SetText("|cFF40FF00Yes");
		elseif(f.state == "no") then
			f.text:SetText("|cFFFF4000No");
		else
			f.text:SetText("|cFFFFC000-");
		end
	end
	
	f.GetValue = function(this)
		return f.state;
	end

	return f;
end

-- Populate Conditions with the data of a rotation and install a save button handler
function Conditions:Populate(spec, id, count, data, saveHandler, cancelHandler)
	self.spec = spec;
	
	local name, _, icon = GetSpellInfo(id);

	self.icon:SetTexture(icon);
	self.nameText:SetText(name);
	self.countText:SetText(count);
	self.manaLabel:SetText(self.spec:GetCostRangeLabel());
	self.data = data;

	local t = deepCopy(self.spec.spellinfo:GetEffects());
	self.effects = { };
	for id,_ in pairs(t) do
		self.effects[#self.effects+1] = id;
	end

	table.sort(self.effects, function(a, b)
		local ea = t[a];
		local eb = t[b];
		
		if((ea.debuff) and (eb.debuff)) or ((not ea.debuff) and (not eb.debuff)) then
			return a < b;
		end
		
		return (not ea.debuff) and (eb.debuff);
	end);

	self.saveHandler = saveHandler;
	self.cancelHandler = cancelHandler;

	-- Populate effect icons and controls
	for i=1,#self.effects do
		local id = self.effects[i];
		
		self.icons[i]:Show();
		self.icons[i]:SetTexture(t[id].icon);
		
		if(t[id].debuff) then
			self.borders[i]:Show();
			self.borders[i]:SetVertexColor(0.5, 0, 1, 1);
		else
			self.borders[i]:Hide();
			self.borders[i]:SetVertexColor(0.25, 0.25, 1, 1);
		end
		
		if(t[id].label) then
			self.labels[i]:Show();
			self.labels[i]:SetText(t[id].label);
		else
			self.labels[i]:Hide();
		end
		
		-- Active buff/debuff flag
		self.active[i]:Show();
		local state;
		local buffs = data.buffs or EMPTY;
		if(buffs[id] == true) then
			state = "yes";
		elseif(buffs[id] == false) then
			state = "no";
		else
			state = nil;
		end
		self.active[i]:SetValue(state);
		
		-- Remaining time
		self.minTime[i]:Show();
		local remainingMin = data.remainingMin or EMPTY;
		self.minTime[i]:SetText(remainingMin[id] or "");

		self.maxTime[i]:Show();
		local remainingMax = data.remainingMax or EMPTY;
		self.maxTime[i]:SetText(remainingMax[id] or "");
		
		-- Stack
		self.minStack[i]:Show();
		local stackMin = data.stackMin or EMPTY;
		self.minStack[i]:SetText(stackMin[id] or "");

		self.maxStack[i]:Show();
		local stackMax = data.stackMax or EMPTY;
		self.maxStack[i]:SetText(stackMax[id] or "");
	end
	
	self.hopo:SetText(data.hopo or "");
	self.manaMin:SetText(data.manaMin or "");
	self.manaMax:SetText(data.manaMax or "");
	
	-- Hide all unnecessary controls and change the size of the frame
	for i=#self.effects+1,MAX_EFFECTS do
		self.icons[i]:Hide();
		self.borders[i]:Hide();
		self.labels[i]:Hide();
		self.active[i]:Hide();
		self.minTime[i]:Hide();
		self.maxTime[i]:Hide();
		self.minStack[i]:Hide();
		self.maxStack[i]:Hide();
	end
	self:SetWidth(64 + 24/2 + 8 + (#self.effects)*(24+8) + 16);
end

Conditions:Construct();
