Palette = CreateFrame("Frame", "AngeleDeiPalette", Sequence);

local ICON_GAP = 6;
local ICON_SIZE = 24;

-- Create the palette frame but don't populate it
function Palette:Construct()
	local backdrop = {
		--bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background", 
		--edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
		--bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", 
		--edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",		
		tile = true,
		tileSize = 16,
		edgeSize = 16,
		insets = {
			left = 4,
			right = 4,
			top = 4,
			bottom = 4
		}
	};
	
	self.icons = { };
	for y=0,5 do
		self.icons[y] = { };
		for x=0,2 do
			local icon = self:CreateTexture(self:GetName() .. "-" .. y .. "." .. x);
			self.icons[y][x] = icon;
			icon:SetWidth(24);
			icon:SetHeight(24);
			local x0 = ICON_GAP+2+x*(ICON_SIZE+ICON_GAP);
			local y0 = -ICON_GAP-2-y*(ICON_SIZE+ICON_GAP);
			icon.x0 = x0;
			icon.y0 = y0;
			icon.xc = x0 + ICON_SIZE/2;
			icon.yc = y0 - ICON_SIZE/2;
			icon:SetPoint("TOPLEFT", self, "TOPLEFT", x0, y0);			
			icon:SetTexture("Interface\\Icons\\Spell_Nature_Polymorph_Cow");
		end
	end

	self:SetBackdrop(backdrop);
	self:SetBackdropColor(0, 0, 0, 1);
	self:SetFrameLevel(40);
	self:SetWidth(200);
	self:SetHeight(200);
	self:SetPoint("LEFT", Sequence, "RIGHT", 4, 0);
	
	self.tooltip = self:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
	self.tooltip:SetPoint("LEFT", self, "RIGHT", 4, 0);
	self.tooltip:SetText("?");
	self.tooltip:SetShadowColor(1, 1, 0, 0.15);
	self.tooltip:SetShadowOffset(1, -1);
	self.tooltip:Hide();
	
	self:EnableMouse();
	self:SetScript("OnUpdate", function()
		self:ShowTooltip();
		if(self.dragged.isMoving) then
			if(Sequence:IsMouseOver()) then
				Sequence:PaletteItemDragged();
			end
		end
	end);
	
	self.dragged = CreateFrame("Frame", "AngeleDeiPaletteDragged", Palette);
	self.dragged:SetWidth(32);
	self.dragged:SetHeight(32);
	self.dragged:ClearAllPoints();
	self.dragged:SetPoint("CENTER", Palette, "CENTER", 0, 0);
	self.dragged.texture = self.dragged:CreateTexture(nil);
	self.dragged.texture:SetAllPoints(self.dragged);
	self.dragged.texture:SetTexture("Interface\\Icons\\Spell_Nature_Polymorph_Cow");
	self.dragged:SetAlpha(0.6);
	self.dragged:SetMovable(true);
	self.dragged:SetFrameStrata("TOOLTIP");
	self.dragged:Hide();

	self:SetScript("OnMouseDown", function(x, button)
		--print("OnMouseDown");
		if(button == "LeftButton") then
			local icon = self:GetMouseOverIcon();
			if(icon == nil) then
				return;
			end
		
			self.dragged:ClearAllPoints();
			local x, y = GetCursorPosition();
			local s = UIParent:GetEffectiveScale();
			self.dragged:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x/s + ICON_SIZE/2, y/s - ICON_SIZE/2);
			self.dragged.texture:SetTexture(icon.texture);
			self.dragged.id = icon.id;
			self.dragged.name = icon.name;
			self.dragged:Show();			
			self.dragged:StartMoving();
			self.dragged:EnableMouse(false);
			self.dragged.isMoving = true;
		end
	end);
	
	self:SetScript("OnMouseUp", function(x, button)
		--print("OnMouseUp");
		if(button == "LeftButton") then
			if(self.dragged.isMoving) then
				self.dragged:StopMovingOrSizing();
				self.dragged.isMoving = false;
				self.dragged:Hide();
				
				if(Sequence:IsMouseOver()) then
					Sequence:PaletteItemDropped(self.dragged.id);
				end
				
				self.dragged.id = nil;
			end
		end
	end);

	-- Let the parent control our visibility
	self:Show();
end

-- Show a palette using the given map of spell IDs (keys)
function Palette:Populate(spec)
	self.spec = spec;
	
	local m = self.spec.spellinfo:GetSpells();
	self.ids = { };
	for id,_ in pairs(m) do
		if(id ~= IDLE) then
			self.ids[#self.ids+1] = id;
			--print("[Palette:Populate] " .. #self.ids .. ": " .. id .. ", " .. GetSpellInfo(id));
		end
	end	
	table.sort(self.ids);
	local n = #self.ids;
	local xx, yy = -1, -1;
	
	for y=0,5 do
		for x=0,2 do
			local i = x + y*3 + 1;
			--print("(" .. x .. "," .. y .. ") " .. i .. "/" .. n .. ": " .. (list[i] or "?"));
			local icon = self.icons[y][x];
			if(i <= n) then
				local name,_,t = GetSpellInfo(self.ids[i]);
				icon:SetTexture(t);
				icon:Show();
				icon.name = name;
				icon.texture = t;
				icon.id = self.ids[i];
				
				xx = math.max(xx, x);
				yy = math.max(yy, y);
			else
				icon:Hide();
			end
		end
	end
	
	self:SetWidth(ICON_GAP+4+(xx+1)*(24+ICON_GAP));
	self:SetHeight(ICON_GAP+4+(yy+1)*(24+ICON_GAP));

	self.cover = CreateFrame("Frame", self:GetName() .. "Cover", self);
	self.cover:SetAllPoints();
	self.cover:EnableMouse(true);
	self.cover:SetFrameLevel(self:GetFrameLevel()+20);
	self.cover:Hide();
end

-- Display a tooltip
function Palette:ShowTooltip()
	if(GetMouseFocus() ~= self) then
		return;
	end
	
	local icon = self:GetMouseOverIcon();
	if(icon == nil) then
		self.tooltip:Hide();
	else
		self.tooltip:SetText(icon.name);
		self.tooltip:SetPoint("LEFT", self, "TOPRIGHT", 4, icon.yc);
		self.tooltip:Show();
	end
end

-- Return the icon we are mosuing over
function Palette:GetMouseOverIcon()
	if(GetMouseFocus() ~= self) then
		return nil;
	end

	for y=0,5 do
		for x=0,2 do
			local icon = self.icons[y][x];
			if(not icon:IsVisible()) then
				return nil;						-- Ran out of icons
			end
			if(icon:IsMouseOver(0, 0, 0, 0)) then
				return icon, x, y;
			end
		end
	end
	
	return nil;
end

Palette:Construct();
