RotationList = CreateFrame("Frame", "AngeleDeiRotationList", UIParent);

local ROW_HEIGHT = 18;

-- Create the Sequence frame but don't populate it
function RotationList:Construct()
	local backdrop = {
		--bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background", 
		--edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
		--bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", 
		--edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		--bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", 
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		tile = true,
		tileSize = 24,
		edgeSize = 24,
		insets = {
			left = 8,
			right = 8,
			top = 8,
			bottom = 8
		}
	};
	
	self.list = { };
	self.selection = nil;
	
	self:EnableMouse(true);
	self:SetBackdrop(backdrop);
	self:SetBackdropColor(0, 0, 0, 1);
	self:SetFrameLevel(0);
	
	self.closeButton = CreateFrame("Button", self:GetName() .. "Close", self, "UIPanelButtonTemplate");
	self.closeButton:SetText("Close");
	self.closeButton:SetWidth(64);
	self.closeButton:SetHeight(24);
	self.closeButton:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -10, 10);
	self.closeButton:SetScript("OnClick", function()
		self:Hide();
	end);

	self.newButton = CreateFrame("Button", self:GetName() .. "New", self, "UIPanelButtonTemplate");
	self.newButton:SetText("New");
	self.newButton:SetWidth(64);
	self.newButton:SetHeight(24);
	self.newButton:SetPoint("TOPRIGHT", self, "TOPRIGHT", -10, -10);
	self.newButton:SetScript("OnClick", function()
		local r = Rotation:new();
		r.name = "";
		self:EditRotation(r);
	end);

	self.deleteButton = CreateFrame("Button", self:GetName() .. "Delete", self, "UIPanelButtonTemplate");
	self.deleteButton:SetText("Delete");
	self.deleteButton:SetWidth(64);
	self.deleteButton:SetHeight(24);
	self.deleteButton:SetPoint("TOPLEFT", self.newButton, "BOTTOMLEFT", 0, -4);
	self.deleteButton:SetScript("OnClick", function()
		if(self.selection == nil) then
			return;
		end
		
		self.spec:DeleteRotation(self.list[self.selection].rotationName);
		self.selection = nil;
		self:Update();
		self.spec:Save();
	end);

	self.singleButton = CreateFrame("Button", self:GetName() .. "Single", self, "UIPanelButtonTemplate");
	self.singleButton:SetText("Single");
	self.singleButton:SetWidth(64);
	self.singleButton:SetHeight(24);
	self.singleButton:SetPoint("TOPLEFT", self.deleteButton, "BOTTOMLEFT", 0, -4);
	self.singleButton:SetScript("OnClick", function()
		self.spec:SetSingleRotation(self.list[self.selection].rotationName);
		self:Update();
		self.spec:Save();
	end);

	self.aoeButton = CreateFrame("Button", self:GetName() .. "AOE", self, "UIPanelButtonTemplate");
	self.aoeButton:SetText("AoE");
	self.aoeButton:SetWidth(64);
	self.aoeButton:SetHeight(24);
	self.aoeButton:SetPoint("TOPLEFT", self.singleButton, "BOTTOMLEFT", 0, -4);
	self.aoeButton:SetScript("OnClick", function()
		self.spec:SetAOERotation(self.list[self.selection].rotationName);
		self:Update();
		self.spec:Save();
	end);
	
	self.scrollFrame = CreateFrame("ScrollFrame", self:GetName() .. "Scroll", self);
	self.scrollFrame:SetPoint("TOPLEFT", self, "TOPLEFT", 10, -10);
	self.scrollFrame:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -80, 10);
	
	self.scrollable = CreateFrame("Frame", self:GetName() .. "List", self);
	self.scrollable:SetWidth(370);
	self.scrollable:SetHeight(375);
	self.scrollable:Show();
	self.scrollable:EnableMouse(true);
	self.scrollFrame:SetScrollChild(self.scrollable);
	self.scrollable:SetScript("OnMouseDown", function(x, button)
		self.selection = self:GetSelection();
		self:UpdateSelection();
		
		if(button == "RightButton") then
			if(self.selection == nil) then
				return;
			end
			if(self.list[self.selection] == nil) then
				return;
			end
			local name = self.list[self.selection].rotationName;
			self:EditRotation(self.spec:GetRotation(name));
		end
	end);
	
	self.slider = CreateFrame("Slider", self:GetName() .. "Slider", self.scrollFrame, "UIPanelScrollBarTemplate");
	self.slider:SetPoint("TOPRIGHT", self.scrollFrame, "TOPRIGHT", 0, -16);
	self.slider:SetPoint("BOTTOMLEFT", self. scrollFrame, "BOTTOMRIGHT", -16, 16);
	self.slider.background = self.slider:CreateTexture(nil, "BACKGROUND");
	self.slider.background:SetAllPoints();
	self.slider.background:SetTexture(0.5, 0.5, 0.5, 0.15);
	
	self.slider:SetMinMaxValues(0, 200);
	self.slider:SetValue(0);

	self:SetWidth(480);
	self:SetHeight(240);
	self:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
	self:Hide();
	
	self.selector = self.scrollable:CreateTexture(nil, "ARTWORK");
	self.selector:SetDrawLayer("ARTWORK", 1);
	self.selector:SetWidth(self.scrollable:GetWidth());
	self.selector:SetHeight(ROW_HEIGHT);
	self.selector:SetTexture(1, 0.75, 0.25, 0.25);
	self.selector:SetPoint("TOPLEFT", self.scrollable, "TOPLEFT");
	self.selector:Hide();
	
	self.cover = CreateFrame("Frame", self:GetName() .. "Cover", self);
	self.cover:SetAllPoints();
	self.cover:EnableMouse(true);
	self.cover:SetFrameLevel(self:GetFrameLevel()+20);
	self.cover:Hide();
end

-- Populate sequence with the data of a rotation and install a save button handler
function RotationList:Populate(spec)
	self.spec = spec;
	
	self:Update();
end

-- Open a sequence window using the given rotation
function RotationList:EditRotation(rotation)
	local name = nil
	if(self.selection ~= nil) then
		name = self.list[self.selection].rotationName;
	end
	
	Sequence:Populate(
		self.spec, 
		rotation,
		function(info)
			-- Save handler: update the rotation
			local saved = self.spec:SaveRotation(info);
			if(saved) then
				self:Update();
				self:SetSelectionByName(name);
				self:UpdateSelection();
				self:SetAlpha(1);
				self.cover:Hide();
				self.spec:Save();
			else
				-- Not saved...
			end
			return saved;
		end,
		function(info)
			-- Cancel handler: do nothing
			self:SetAlpha(1);
			self.cover:Hide();
		end);
	self:SetAlpha(0.5);
	self.cover:Show();
	Sequence:Show();
end

-- Update the contents of the list, the selection, the buttons and the scroll bar
function RotationList:Update()
	local m = self.spec:GetRotations();
	local v = self.slider:GetValue();
	local _, vMax = self.slider:GetMinMaxValues();
	
	-- Sort the names alphabetically
	local sorted = { };
	for _,r in pairs(m) do
		sorted[#sorted+1] = r;
	end
	
	table.sort(sorted, function(x,y)
		local xx, yy = x:GetName(), y:GetName();
		if(x:IsLocked()) then
			xx = " " .. xx;
		end
		if(y:IsLocked()) then
			yy = " " .. yy;
		end
		
		return xx < yy;
	end);
	
	-- Create and initialize font strings for each name
	local i = 1;
	for _,r in ipairs(sorted) do
		local name = r:GetName();
		
		if(i > #self.list) then
			local s = self.scrollable:CreateFontString(nil, "ARTWORK", "GameFontHighlight");
			s:SetPoint("TOPLEFT", self.scrollable, "TOPLEFT", 0, -(i-1)*ROW_HEIGHT-2);
			s:SetWidth(self.scrollable:GetWidth());
			s:SetHeight(ROW_HEIGHT);
			s:SetJustifyH("LEFT");
			s:SetJustifyV("MIDDLE");
			s:SetDrawLayer("ARTWORK", 10);
			self.list[i] = s;
		end
		local line = self.list[i];
		
		local n = name;
		if(r:IsLocked()) then
			n = "|cFFFFC040" .. n; -- .. "  |TInterface\\Icons\\Ability_Paladin_DivineStorm:20:20:0:0|t"; --"  |TInterface\\Icons\\Ability_SteelMelee:16:16:0:0|t";
		end
		
		if(r:GetName() == self.spec:GetSingleRotation()) then
			n = "|TInterface\\AddOns\\AngeleDei\\images\\marker-single20:20:20:0:0|t" .. n;
		elseif(r:GetName() == self.spec:GetAOERotation()) then
			n = "|TInterface\\AddOns\\AngeleDei\\images\\marker-aoe20:20:20:0:0|t" .. n;
		else
			n = "|Tnonexistent-texture:20:20:0:0|t" .. n;
		end
		
		line:SetText(n);
		line.rotationName = name;
		line:Show();
		i = i + 1;
	end
	
	local height = ROW_HEIGHT*(i-1);
	self.scrollable:SetHeight(height);
	
	-- Hide all unused strings
	while(i <= #self.list) do
		self.list[i]:Hide();
		i = i + 1;
	end
	
	self:UpdateSelection();
	
	-- Update the scroll bar
	if(height <= self.scrollFrame:GetHeight()) then
		self.slider:SetMinMaxValues(0, 0);
		self.slider:SetValue(0);
		self.slider:Hide();
	else
		local newMax = height - self.scrollFrame:GetHeight();
		local newV = v / vMax * newMax;
		self.slider:SetMinMaxValues(0, newMax);
		self.slider:SetValue(newV);
		self.slider:Show();
	end	
end

-- Update the selection highlight
function RotationList:UpdateSelection()
	-- Sanity check
	if(self.selection) and ((self.selection < 1) or (self.selection > #self.list)) then
		self.selection = nil;
	end
	
	if(self.selection == nil) then
		self.selector:Hide();
		self.deleteButton:Disable();
		self.singleButton:Disable();
		self.aoeButton:Disable();
	else
		local name = self.list[self.selection].rotationName;
		local r = self.spec:GetRotation(name);
		self.selector:Show();
		self.selector:SetPoint("TOPLEFT", self.scrollable, "TOPLEFT", 0, -(self.selection-1)*ROW_HEIGHT);
		
		--print("Selection=" .. self.selection .. ", name='" .. name .. "', r.name='" .. r:GetName() .. "', r.locked=" .. (r:IsLocked() and "true" or "false"));
		
		if (r:IsLocked()) then
			self.deleteButton:Disable();
		else
			self.deleteButton:Enable();
		end
		
		if(name == self.spec:GetSingleRotation()) then
			self.singleButton:Disable();
		else
			self.singleButton:Enable();
		end
		
		if(name == self.spec:GetAOERotation()) then
			self.aoeButton:Disable();
		else
			self.aoeButton:Enable();
		end
	end
	
end

-- Get the selection index (nil if nothing is selected)
function RotationList:GetSelection()
	for i=1,#self.list do
		if(self.list[i]:IsMouseOver(0, 0, 0, 0)) then
			return i;
		end
	end
	
	return nil;
end

-- Set the current selection by name
function RotationList:SetSelectionByName(name)
	for i=1,#self.list do
		if(self.list[i].rotationName == name) then
			self.selection = i;
			return self.selection;
		end
	end
	
	self.selection = nil;
	return nil;
end

RotationList:Construct();
--RotationList:Populate(spec);
--RotationList:Show();
