Sequence = CreateFrame("Frame", "AngeleDeiSequence", UIParent);

local ICON_GAP = 6;
local ICON_SIZE = 32;
local MAX_ICONS = 15;

-- Create the Sequence frame but don't populate it
function Sequence:Construct()
	local backdrop = {
		--bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background", 
		--edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
		--bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", 
		--edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		--bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", 
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		tile = true,
		tileSize = 24,
		edgeSize = 24,
		insets = {
			left = 8,
			right = 8,
			top = 8,
			bottom = 8
		}
	};
	
	self:EnableMouse(true);
	self:SetBackdrop(backdrop);
	self:SetBackdropColor(0, 0, 0, 1);
	self:SetFrameLevel(40);
	
	local w = MAX_ICONS*ICON_SIZE + (MAX_ICONS-1)*ICON_GAP + ICON_GAP*2 + 8*2;
	self:SetWidth(w);
	self:SetHeight(132);
	self:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
	
	self.labelText = self:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	self.labelText:SetPoint("TOPLEFT", 16, -16);
	self.labelText:SetText("Name:");

	self.codeText = self:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	self.codeText:SetPoint("TOPLEFT", self.labelText, "BOTTOMLEFT", 0, -8);
	self.codeText:SetPoint("RIGHT", self.labelText, "RIGHT", 0, 0);
	self.codeText:SetText("Code:");
	
	self.label = CreateFrame("EditBox", "LabelEditBox", self, "InputBoxTemplate");
	--self.label:SetWidth(120);
	self.label:SetHeight(22);
	self.label:SetPoint("LEFT", self.labelText, "RIGHT", 8, 0);
	self.label:SetPoint("RIGHT", self, "RIGHT", -16, 0);
	self.label:SetJustifyH("LEFT");
	self.label:SetMultiLine(false);
	self.label:SetAutoFocus(false);
	self.label:SetScript("OnShow", function(self) self:SetCursorPosition(0); end);
	self.label:SetScript("OnTextChanged", function()
		-- Save the rotation name!
		local t = self.label:GetText();
		self.rotation.name = t and string.trim(t) or "";

		self:UpdateSaveButton();
	end);

	self.code = self:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
	--self.code:SetWidth(120);
	self.code:SetHeight(22);
	self.code:SetJustifyH("LEFT");
	self.code:SetPoint("LEFT", self.codeText, "RIGHT", 8, 0);	
	self.label:SetPoint("RIGHT", self, "RIGHT", -16, 0);
	self.code:SetText("|cFFFFC0C0SotR|cFFFFFFFF3|r | |cFFFFC0C0CS");
	
	self.icons = { };
	self.zones = { };
	self.highlights = { };
	for i=0,MAX_ICONS-1 do
		local x = i*(ICON_GAP + ICON_SIZE) + ICON_GAP + 8;
		
		local x0 = x - ICON_GAP - ICON_SIZE/2;		
		local x1 = x + ICON_SIZE/2;
		
		local icon;
		local highlight;
		if(i < MAX_ICONS) then
			icon = self:CreateTexture(self:GetName() .. "-" .. i);
			icon:SetWidth(ICON_SIZE);
			icon:SetHeight(ICON_SIZE);
			--icon:SetTexture("Interface\\PaperDoll\\Ui-paperdoll-slot-secondaryhand"); 
			--icon:SetAlpha(0.75);
			icon:SetPoint("TOP", self.code, "BOTTOM", x, -4);
			icon:SetPoint("LEFT", self, "LEFT", x, 0);
			icon:SetDrawLayer("ARTWORK", 1);
			self.icons[i+1] = icon;
			
			highlight = self:CreateTexture(self:GetName() .. "-highlight-" .. i);
			highlight:SetTexture("Interface\\AddOns\\AngeleDei\\images\\default");
			highlight:SetWidth(ICON_SIZE/3);
			highlight:SetHeight(ICON_SIZE/3);
			highlight:SetPoint("CENTER", icon, "TOPRIGHT", -ICON_SIZE/16, -ICON_SIZE/16);
			highlight:SetDrawLayer("ARTWORK", 2);
			self.highlights[i+1] = highlight;			
		end
		
		local zone = self:CreateTexture(self:GetName() .. "Zone-" .. i);
		zone:SetWidth(ICON_SIZE);
		zone:SetHeight(ICON_SIZE);
		--zone:SetTexture("Interface\\PaperDoll\\Ui-paperdoll-slot-secondaryhand"); 
		zone:SetPoint("TOP", self.icons[1], "TOP", 0, 2);
		zone:SetPoint("BOTTOM", self.icons[1], "BOTTOM", 0, -2);
		zone:SetPoint("LEFT", self, "LEFT", x0, 0);
		zone:SetPoint("RIGHT", self, "LEFT", x1, 0);
		zone:SetAlpha(0);
		zone:SetTexture(1, 0, 0);
		self.zones[i+1] = zone;
	end
	
	self.saveButton = CreateFrame("Button", self:GetName() .. "OK", self, "UIPanelButtonTemplate");
	self.saveButton:SetText("Save");
	self.saveButton:SetWidth(64);
	self.saveButton:SetHeight(24);
	self.saveButton:SetPoint("TOPLEFT", self.icons[1], "BOTTOMLEFT", 0, -8);
	self.saveButton:SetScript("OnClick", function()
		local closeMe = true;
		if(self.saveHandler ~= nil) then
			closeMe = self.saveHandler(self.rotation);
		end
		
		if(closeMe) then
			self.saveHandler = nil;
			self.cancelHandler = nil;
			self:Hide();
		end
	end);

	self.cancelButton = CreateFrame("Button", self:GetName() .. "Cancel", self, "UIPanelButtonTemplate");
	self.cancelButton:SetText("Cancel");
	self.cancelButton:SetWidth(64);
	self.cancelButton:SetHeight(24);
	self.cancelButton:SetPoint("TOPLEFT", self.saveButton, "TOPRIGHT", 6, 0);
	self.cancelButton:SetScript("OnClick", function()
		if(self.cancelHandler ~= nil) then
			self.cancelHandler();
		end

		self.saveHandler = nil;
		self.cancelHandler = nil;
		self:Hide();
	end);
	
	self.caret = self:CreateTexture(self:GetName() .. "Caret");
	self.caret:SetWidth(38);
	self.caret:SetHeight(38);
	self.caret:SetTexCoord(0, 0.59375, 0, 0.59375);
	self.caret:SetTexture("Interface\\AddOns\\AngeleDei\\images\\caret.tga"); 
	self.caret:Hide();
	
	self.dragged = CreateFrame("Frame", "AngeleDeiSequenceDragged", Sequence);
	self.dragged:SetWidth(32);
	self.dragged:SetHeight(32);
	self.dragged:ClearAllPoints();
	self.dragged:SetPoint("CENTER", Sequence, "CENTER", 0, 0);
	self.dragged.texture = self.dragged:CreateTexture(nil);
	self.dragged.texture:SetAllPoints(self.dragged);
	self.dragged.texture:SetTexture("Interface\\Icons\\Spell_Nature_Polymorph_Cow");
	self.dragged:SetAlpha(0.6);
	self.dragged:SetMovable(true);
	self.dragged:SetFrameStrata("TOOLTIP");
	self.dragged:Hide();

	self:SetScript("OnUpdate", function()
		local blink = (mod(math.floor(GetTime()*40), 10) < 6);
		
		-- Blinking caret
		if(self.caretZone == nil) then
			self.caret:Hide();
		else			
			self.caret:SetPoint("LEFT", self.zones[self.caretZone], "CENTER", -4, 0);
			if(blink) then
				self.caret:Show();
			else
				self.caret:Hide();
			end
		end
		
		-- Blinking icon
		if(self.movedIcon) then
			local movedIcon = self.icons[self.movedIcon];
			local movedHighlight = self.highlights[self.movedIcon];
			if(blink) then
				local z = self:GetMouseOverZone();
				if(z) then	-- (moving: regular)
					movedIcon:SetVertexColor(1, 1, 1);
					
					-- Blink the caret too to indicate where the icon will be moving to
					self.caret:SetPoint("LEFT", self.zones[z], "CENTER", -4, 0);
					if(blink) then
						self.caret:Show();
					else
						self.caret:Hide();
					end
				else		-- (deleting: red)
					movedIcon:SetVertexColor(1, 0.15, 0.15);
					self.caret:Hide();
				end
					
				movedIcon:Show();
				if(self.rotation:get(self.movedIcon):HasConditions()) then
					movedHighlight:Show();
				else
					movedHighlight:Hide();
				end
			else
				movedIcon:Hide();
				movedHighlight:Hide();
			end
		end			
	end);
	
	self:SetScript("OnMouseDown", function(x, button)
		-- LMB: drag
		if(button == "LeftButton") then
			local i = self:GetMouseOverIcon();
			if(i == nil) then
				return;
			end
			
			self.movedIcon = i;			
			local icon = self.icons[self.movedIcon];
		
			self.dragged:ClearAllPoints();
			local x, y = GetCursorPosition();
			local s = UIParent:GetEffectiveScale();
			self.dragged:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x/s + ICON_SIZE/2, y/s - ICON_SIZE/2);
			self.dragged.texture:SetTexture(icon:GetTexture());
			self.dragged:Show();			
			self.dragged:StartMoving();
			self.dragged:EnableMouse(false);
			self.dragged.isMoving = true;
			return;
		end
		
		-- RMB: conditions dialog
		if(button == "RightButton") then
			local i = self:GetMouseOverIcon();
			if(i == nil) then
				return;
			end
			
			self.edited = i;
			self:SetAlpha(0.65);
			Palette:SetAlpha(0.65);
			self.cover:Show();
			Palette.cover:Show();
			
			-- Open the Conditions dialog
			local re = self.rotation:get(i);
			Conditions:Populate(
				self.spec,
				re.id, 
				i .. " of " .. self.rotation:size(),
				re:Save(),
				function(info)		-- Save handler
					self.rotation:set(self.edited, RotationElement:Load(info));
					self:SetAlpha(1);
					Palette:SetAlpha(1);
					self.cover:Hide();
					Palette.cover:Hide();
					self:Update();
				end,
				function()			-- Cancel handler
					self:SetAlpha(1);
					Palette:SetAlpha(1);
					Palette.cover:Hide();
					self.cover:Hide();
				end);
			
			Conditions:Show();
			return;
		end
	end);
	
	self:SetScript("OnMouseUp", function(x, button)
		--print("OnMouseUp");
		if(button == "LeftButton") then
			if(self.dragged.isMoving) then
				self.dragged:StopMovingOrSizing();
				self.dragged.isMoving = false;
				self.dragged:Hide();
				
				local z = self:GetMouseOverZone();
				if(z) then	-- (move the icon)
					if(z > self.movedIcon) then
						z = z - 1;
					end
					local x = self.rotation:remove(self.movedIcon);
					--print("Removing icon at " .. self.movedIcon .. ", inserting it at " .. z);
					self.rotation:insert(z, x);
				else		-- (delete the icon)
					self.rotation:remove(self.movedIcon);
				end

				self.movedIcon = nil;
				self:Update();				
			end
		end
	end);
	
	self.cover = CreateFrame("Frame", self:GetName() .. "Cover", self);
	self.cover:SetAllPoints();
	self.cover:EnableMouse(true);
	self.cover:SetFrameLevel(self:GetFrameLevel()+20);
	self.cover:Hide();
	
	self:Hide();
end

-- Update the save button based on the state of the window
function Sequence:UpdateSaveButton()
		local t = self.label:GetText();
		if(t) then
			t = string.trim(t);
		end
		
		local r = self.spec:GetRotation(t);		
		
		if(t == nil) or (t == "") or ((r ~= nil) and (r:IsLocked())) or (self.rotation:size() == 0) then
			self.saveButton:Disable();
		else
			self.saveButton:Enable();
		end
end

-- Populate sequence with the data of a rotation and install a save button handler
function Sequence:Populate(spec, r, save, cancel)
	self.spec = spec;
	self.rotation = deepCopy(r);
	self.rotation.locked = false;
	self.label:SetText(self.rotation:GetName());
	self.saveHandler = save;
	self.cancelHandler = cancel;
	self:Update();
	
	Palette:Populate(self.spec);
end

-- Update the icons according to the current state of the rotation
function Sequence:Update()
	local n = self.rotation:size();
	for i=1,MAX_ICONS do
		local icon = self.icons[i];
		icon:SetVertexColor(1, 1, 1);
		icon:Show();
		if(i > n) then
			icon:SetTexture("Interface\\PaperDoll\\Ui-paperdoll-slot-secondaryhand"); 
			icon:SetAlpha(0.75);
			self.highlights[i]:Hide();
		else
			local re = self.rotation:get(i);
			local h = self.highlights[i];
			if(re:HasConditions()) then
				h:Show();
			else
				h:Hide();
			end

			icon:SetTexture(re:GetTexture()); 
			icon:SetAlpha(1);
		end		
	end
	
	local code = self.rotation:GetCode(self.spec.spellinfo, true);
	self.code:SetText(code);
	self:UpdateSaveButton();
end

-- Figure out which zone we are mousing over
function Sequence:GetMouseOverZone()
	for i=1,MAX_ICONS do
		if(self.zones[i]:IsMouseOver()) then
			return math.min(i, self.rotation:size()+1);
		end
	end
	return nil;
end

-- Figure out which icon we are picking up
function Sequence:GetMouseOverIcon()
	for i=1,MAX_ICONS do
		if(self.icons[i]:IsMouseOver()) then
			if(i > self.rotation:size()) then
				return nil;
			else
				return i;
			end
		end
	end
	return nil;
end

-- This function is called when an icon is dragged from the palette onto the sequence row
function Sequence:PaletteItemDragged()
	if(self.rotation:size() >= MAX_ICONS) then
		self.caretZone = nil;
	else
		self.caretZone = self:GetMouseOverZone();
	end
end

-- This function is called when an icon is dropped from the palette onto the sequence row
function Sequence:PaletteItemDropped(id)
	self.caretZone = nil;
	
	-- Don't allow drops from the palette if the sequence is already full
	if(self.rotation:size() >= MAX_ICONS) then
		return;
	end
	
	-- Are we actually dropping onto the sequence row?
	local z = self:GetMouseOverZone();
	if(z == nil) then
		return;
	end
	
	-- Looks like we are! Create a new rotation
	local re = RotationElement:new(id);
	self.rotation:insert(z, re);
	--print("New rotation size: " .. self.rotation:size());
	self:Update();
end

Sequence:Construct();
--local spec = new(ProtectionSpec);
--local r = spec:GetRotations()[1];
--Sequence:Populate(spec, r);
--Sequence:Show();
