-- Create the Display panel
function Options:CreateDisplayPanel()
	local panel = Options:CreatePanel(TITLE_DISPLAY, SUBTITLE_DISPLAY);
	self.display = panel;
	
	-- Holy Power
	local hopoLabel = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	hopoLabel:SetPoint("TOPLEFT", panel.subtitle, "BOTTOMLEFT", 0, -4);
	hopoLabel:SetText(SECTION_HOPO);
	
	panel.fadeOut = self:CreateCheckButton(LABEL_FADEOUT, panel);
	panel.fadeOut:SetPoint("TOPLEFT", hopoLabel, "BOTTOMLEFT", 0, -4);

	panel.blink = self:CreateCheckButton(LABEL_BLINK, panel);
	panel.blink:SetPoint("TOPLEFT", panel.fadeOut, "BOTTOMLEFT", 0, 4);
	
	panel.style = self:CreateStyleSlider(LABEL_HOPO_STYLE, panel);
	panel.style:SetPoint("TOPLEFT", panel.blink, "BOTTOMLEFT", 0, -16);
	
	-- Vengeance
	local vengLabel = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	vengLabel:SetPoint("TOPLEFT", panel.style, "BOTTOMLEFT", 0, -16);
	vengLabel:SetText(SECTION_VENGEANCE);

	panel.vengeanceEnabled = self:CreateCheckButton(LABEL_VENGEANCE_ENABLE, panel);
	panel.vengeanceEnabled:SetPoint("TOPLEFT", vengLabel, "BOTTOMLEFT", 0, -4);

	panel.vengeanceStyle = self:CreateStyleSlider(LABEL_VENGEANCE_STYLE, panel);
	panel.vengeanceStyle:SetPoint("TOPLEFT", panel.vengeanceEnabled, "BOTTOMLEFT", 0, -16);

	-- Other
	local otherLabel = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	otherLabel:SetPoint("TOPLEFT", panel.vengeanceStyle, "BOTTOMLEFT", 0, -16);
	otherLabel:SetText(SECTION_MISC_DISPLAY);

	panel.showRotation = self:CreateCheckButton(LABEL_SHOW_ROTATION, panel);
	panel.showRotation:SetPoint("TOPLEFT", otherLabel, "BOTTOMLEFT", 0, -4);

	panel.shiftGCD = self:CreateCheckButton(LABEL_SHIFT_GCD, panel);
	panel.shiftGCD:SetPoint("TOPLEFT", panel.showRotation, "BOTTOMLEFT", 0, 4);
	panel.shiftGCD:SetScript("PostClick", function()
		Options:UpdateShiftSpeed();
	end);

	panel.shiftSpeed = self:CreateNumericSlider(LABEL_SHIFT_SPEED, panel, 0.7, 2.0, 0.05, "%2.1f", "%3.2f");
	panel.shiftSpeed:SetPoint("TOPLEFT", panel.shiftGCD, "BOTTOMLEFT", 0, -16);

	-- Populate and save handlers
	panel.Populate = function(self, config)
		Options:PopulateDisplayPanel(config);
	end
	panel.Save = function(self, config)
		Options:SaveDisplayPanel(config)
	end

	return panel;
end

-- Populate the contents of the Display panel
function Options:PopulateDisplayPanel(config)
	local panel = self.display;
	
	panel.style:SetString(config.style);	
	panel.fadeOut:SetChecked(config.fadeOut);
	panel.blink:SetChecked(config.blink);
	
	panel.vengeanceEnabled:SetChecked(config.vengeanceEnabled);
	panel.vengeanceStyle:SetString(config.vengeanceStyle);
	
	panel.showRotation:SetChecked(config.showRotation);
	panel.shiftGCD:SetChecked(config.shiftGCD);
	panel.shiftSpeed:SetValue(config.shiftSpeed);	

	self:UpdateShiftSpeed();
end

-- Save the contents of the Display panel
function Options:SaveDisplayPanel(config)
	local panel = self.display;
	
	config.style = panel.style:GetString();
	config.fadeOut = panel.fadeOut:GetChecked() and true or false;
	config.blink = panel.blink:GetChecked() and true or false;

	config.vengeanceEnabled = panel.vengeanceEnabled:GetChecked() and true or false;
	config.vengeanceStyle = panel.vengeanceStyle:GetString();

	config.showRotation = panel.showRotation:GetChecked() and true or false;
	config.shiftGCD = panel.shiftGCD:GetChecked() and true or false;
	config.shiftSpeed = panel.shiftSpeed:GetValue();
end

-- Update the appearance of the "shift speed" slider bar based on the "shift GCD" checkbox
function Options:UpdateShiftSpeed()
	local panel = self.display;
	
	if(panel.shiftGCD:GetChecked()) then
		panel.shiftSpeed:SetAlpha(0.35);
		panel.shiftSpeed:Disable();
	else
		panel.shiftSpeed:SetAlpha(1.0);
		panel.shiftSpeed:Enable();
	end
end