Options = CreateFrame("Frame", "AngeleDeiOptionsFrame", UIParent)

-- Bootstrap the configuration frame
local f = CreateFrame("Frame", nil, InterfaceOptionsFrame)
f:SetScript("OnShow", function(self)
	Options:Construct();
	--Options:Populate();
	f:SetScript("OnShow", nil);
end);

local styleCodes = { "default", "red", "orange", "golden", "yellow", "lime", "green", "cyan", "blue", "purple", "magenta", "white", "stoplight" };

-- Set the configuration used by this interface. This is called immediately after config info is loaded
function Options:SetConfig(config)
	self.config = config;
end

-- Set the callback invoked when the configuration is saved
function Options:SetCallback(f)
	self.save = f;
end

-- Create a new section
function Options:CreateSection(name, parent, height)
	local section = CreateFrame("Frame", self:GetName() .. "|" .. parent:GetName() .. "|" .. name, parent, "OptionsBoxTemplate");
	section:SetBackdropBorderColor(0.75, 0.75, 0.75);
	section:SetBackdropColor(0.15, 0.15, 0.15, 0.5);
	_G[section:GetName() .. "Title"]:SetText(name);
	_G[section:GetName() .. "Title"]:SetFontObject("GameFontNormalSmall");
	
	section:SetPoint("LEFT", parent, "LEFT", 8, 0);
	section:SetPoint("RIGHT", parent, "RIGHT", -8, 0);
	section:SetHeight(height);

	return section;
end

-- Create an interface options panel
function Options:CreatePanel(title, subtitle)
	local frame = CreateFrame("Frame", "AngeleDeiOptionsFrame." .. title, UIParent);
	
	frame.name = title;
	frame.parent = self.name;
	InterfaceOptions_AddCategory(frame);
	self:CreateTitle(frame, title, subtitle);

	return frame;
end

-- Create a check button
function Options:CreateCheckButton(name, parent)
	local button = CreateFrame("CheckButton", parent:GetName() .. name, parent, "InterfaceOptionsCheckButtonTemplate")
	_G[button:GetName() .. "Text"]:SetText(name)
	return button
end

-- Create a slider
function Options:CreateSlider(name, parent, minValue, maxValue, step)
	local frame = CreateFrame("Slider", parent:GetName() .. name, parent, "OptionsSliderTemplate")
	frame:SetPoint("LEFT", parent, "LEFT", 32, 0);
	frame:SetPoint("RIGHT", parent, "RIGHT", -32, 0);
	frame:SetMinMaxValues(minValue, maxValue);
	frame:SetValueStep(step);
	frame:EnableMouseWheel(true);
	_G[frame:GetName() .. "Low"]:SetText(""); -- .. minValue);
	_G[frame:GetName() .. "High"]:SetText(""); -- .. maxValue);
	_G[frame:GetName() .. "Text"]:SetText(name);
	_G[frame:GetName() .. "Text"]:SetFontObject("GameFontHighlightSmall")
	_G[frame:GetName() .. "Text"]:ClearAllPoints()
	_G[frame:GetName() .. "Text"]:SetPoint("BOTTOMLEFT", frame, "TOPLEFT")
	
	local value = frame:CreateFontString(nil, "BACKGROUND");
	value:SetFontObject("GameFontNormalSmall");
	value:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", 0, 0);
	frame.value = value;
	
	--BlizzardOptionsPanel_Slider_Enable(frame);	--colors the slider properly
	return frame;
end

-- Create a numeric slider
function Options:CreateNumericSlider(name, parent, minValue, maxValue, step, endFormat, valueFormat)
	local slider = self:CreateSlider(name, parent, minValue, maxValue, step);
	
	local minLabel = slider:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
	minLabel:SetText(format(endFormat, minValue));
	minLabel:SetPoint("TOPLEFT", slider, "BOTTOMLEFT", 0, 0);
	local maxLabel = slider:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
	maxLabel:SetText(format(endFormat, maxValue));
	maxLabel:SetPoint("TOPRIGHT", slider, "BOTTOMRIGHT", 0, 0);
	slider:SetScript("OnValueChanged", function(self)
		self.value:SetText(format(valueFormat, self:GetValue()));
	end);
	
	return slider;
end

-- Create a style slider
function Options:CreateStyleSlider(name, parent)
	local slider = Options:CreateSlider(name, parent, 1, #STYLES, 1);
	slider:SetScript("OnValueChanged", function(self)
		self.value:SetText(STYLES[self:GetValue()]);
	end);
	
	slider.SetString = function(self, s)
		local n = indexOf(styleCodes, s);
		self:SetValue(n);
	end
	
	slider.GetString = function(self)
		local s = styleCodes[self:GetValue()];
		return s;
	end

	return slider;
end

-- Create a title and a subtitle
function Options:CreateTitle(frame, title, subtitle)
	frame.title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
	frame.title:SetPoint("TOPLEFT", 16, -16);
	frame.title:SetText(title);

	frame.subtitle = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
	frame.subtitle:SetHeight(24);
	frame.subtitle:SetPoint("TOPLEFT", frame.title, "BOTTOMLEFT", 0, -8);
	frame.subtitle:SetPoint("RIGHT", frame, -32, 0);
	frame.subtitle:SetNonSpaceWrap(true);
	frame.subtitle:SetJustifyH("LEFT");
	frame.subtitle:SetJustifyV("TOP");
	frame.subtitle:SetText(subtitle);
end
