local ICON_SIZE = 32;
local ICON_GAP = 8;
local ICONS_PER_ROW = 10;
local FONT = "Interface\\AddOns\\AngeleDei\\fonts\\russel square lt.ttf";
local TAGS = { "single", "aoe" };

StaticPopupDialogs["KEYSTROKE"] = {
	text = "...",
	button1 = "Accept",
	button2 = "Cancel",
	hasEditBox = true,
	editBoxWidth = 64,
	timeout = 0,
	exclusive = true,
	hideOnEscape = true,
	EditBoxOnEscapePressed = function(self) self:GetParent():Hide() end,
	whileDead = true,
	maxLetters = 5,
	enterClicksFirstButton = 1,
	--showAlert = true,
}

-- Create a Keystroke panel
function Options:CreateKeystrokePanel(spec)
	local _, title = spec:GetDescription();
	local panel = Options:CreatePanel(title, SUBTITLE_KEYSTROKES);
	panel.spec = spec;

	self.keystrokes[#self.keystrokes+1] = panel;
	
	local labels = { LABEL_SINGLE_KEYSTROKES, LABEL_AOE_KEYSTROKES };
	local prev = panel.subtitle;
	local prevOffset = -4;
	
	for i=1,#labels do
		local tag = TAGS[i];
		
		local info = { };		
		panel[tag] = info;
		local icons = { };
		panel[tag .. "Icons"] = icons;

		local label = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
		label:SetPoint("TOPLEFT", prev, "BOTTOMLEFT", 0, prevOffset);
		label:SetText(labels[i]);
		
		-- The list of abilities maintained by this spec
		local m = spec.spellinfo:GetSpells();
		local ids = { };
		for id,_ in pairs(m) do
			if(id ~= IDLE) then
				ids[#ids+1] = id;
			end
		end	
		table.sort(ids);
		
		for i=1,#ids do
			local id = ids[i];
			local icon = panel:CreateTexture(tag .. id);
			icon:SetWidth(ICON_SIZE);
			icon:SetHeight(ICON_SIZE);
			local row = math.floor((i-1)/ICONS_PER_ROW);
			local column = mod(i-1, ICONS_PER_ROW);			
			local x0 = column*(ICON_SIZE+ICON_GAP);
			local y0 = -row*(ICON_SIZE+ICON_GAP) - 12;
			icon:SetPoint("TOPLEFT", label, "BOTTOMLEFT", x0, y0);			
			icon:SetTexture(spec.spellinfo:GetIcon(id));
			icons[id] = icon;
			
			local text = panel:CreateFontString();
			text:SetPoint("TOPRIGHT", icon, "TOPRIGHT", 0, -2);
			text:SetFont(FONT, 12, "OUTLINE");
			text:SetTextColor(1, 1, 1, 1);
			text:SetShadowColor(0, 0, 0, 0.5);
			text:SetShadowOffset(1.5, -1.5);
			text:SetText("");
			icon.text = text;

			if(x0 == 0) then
				prev = icon;
				prevOffset = -16;
			end
		end
	end
	
	panel:SetScript("OnMouseDown", function(x, button)
		local id, tag = self:GetMouseOverIcon(panel);
		if(id == nil) then
			return;
		end
		
		local keystrokes = spec:GetKeystrokes(tag == "aoe");
		
		local info = StaticPopupDialogs["KEYSTROKE"];
		info.text = format(LABEL_DIALOG, GetSpellInfo(id));
		info.OnAccept = function (self, data, data2)
    		local string = string.trim(self.editBox:GetText() or "");
    		panel[tag .. "Icons"][id].text:SetText(string);
    	end
    	
    	local dialog = StaticPopup_Show("KEYSTROKE");
    	local editBox = dialog.editBox;
    	editBox:SetText(keystrokes[id] or "");
		editBox:SetFocus();
		editBox:HighlightText();
		editBox:SetJustifyH("CENTER");
	end);
	
	-- This button will open the rotation list
	panel.editButton = CreateFrame("Button", panel:GetName() .. "|Edit", panel, "UIPanelButtonTemplate");
	panel.editButton:SetText(LABEL_EDIT_ROTATIONS);
	panel.editButton:SetWidth(128);
	panel.editButton:SetHeight(24);
	panel.editButton:SetPoint("TOPLEFT", prev, "BOTTOMLEFT", 0, prevOffset-8);
	panel.editButton:SetScript("OnClick", function()
		InterfaceOptionsFrame:Hide();
		RotationList:Populate(panel.spec);
		RotationList:Show();
	end);

	-- Populate and save handlers
	panel.Populate = function(self, config)
		Options:PopulateKeystrokePanel(panel, config);
	end
	panel.Save = function(self, config)
		Options:SaveKeystrokePanel(panel, config);
	end

	return panel;
end

-- Return the ID and tag of the icon we are mosuing over
function Options:GetMouseOverIcon(panel)
	if(GetMouseFocus() ~= panel) then
		return nil, nil;
	end
	
	for _,tag in ipairs(TAGS) do
		local icons = panel[tag .. "Icons"];
		for id, icon in pairs(icons) do
			if(icon:IsMouseOver(0, 0, 0, 0)) then
				return id, tag;
			end
		end		
	end
	
	return nil, nil;
end

-- Populate the contents of the Keystroke panel
function Options:PopulateKeystrokePanel(panel, config)
	local spec = panel.spec;
	
	for _,tag in ipairs(TAGS) do
		local icons = panel[tag .. "Icons"];
		local keystrokes = spec:GetKeystrokes(tag == "aoe");
		for id, icon in pairs(icons) do
			icon.text:SetText(keystrokes[id] or "");
		end
	end
end

-- Save the contents of the Keystroke panel
function Options:SaveKeystrokePanel(panel, config)
	local spec = panel.spec;
	
	for _,tag in ipairs(TAGS) do
		local icons = panel[tag .. "Icons"];
		local keystrokes = spec:GetKeystrokes(tag == "aoe");
		for id, icon in pairs(icons) do
			local text = string.trim(icon.text:GetText() or "");
			keystrokes[id] = text;
		end
	end
	
	spec:Save();
end
