-- Protection spec for Paladins
ProtectionSpec = new(Spec);

local DEFAULT_1_0 = "Default single target (1.0.x)";
local DEFAULT_1_1 = "Default single target (1.1.x)";
local HIGH_THREAT_1_1 = "High threat single target";
local DEFAULT_AOE = "Default AoE";
local HIGH_THREAT_AOE = "High threat AoE";

-- Construct a single-target rotation used in version 1.0.x
local function createOldSingleTargetRotation(self, name)
	local r = Rotation:new();

	r:add(RotationElement:new(ABILITY_SHOR));
	--
	r:add(RotationElement:new(ABILITY_CS));
	--
	r:add(RotationElement:new(ABILITY_J));
	--
	r:add(RotationElement:new(ABILITY_HOW));
	--
	r:add(RotationElement:new(ABILITY_AS));
	--
	r:add(RotationElement:new(ABILITY_HW));
	--
	local re = RotationElement:new(ABILITY_CON);
	re.manaMin = 75;
	r:add(re);
	
	r:SetName(name);
	r.locked = true;	
	self.rotations[r:GetName()] = r;
end

-- Construct the default simple single-target rotation used in version 1.1.0
local function createNewSingleTargetRotation(self, name)
	local r = Rotation:new();	
	
	r:add(RotationElement:new(ABILITY_SHOR));
	--
	r:add(RotationElement:new(ABILITY_CS));
	--
	r:add(RotationElement:new(ABILITY_AS));
	--
	r:add(RotationElement:new(ABILITY_HOW));
	--
	r:add(RotationElement:new(ABILITY_J));
	--
	local re = RotationElement:new(ABILITY_CON);
	re.manaMin = 75;
	r:add(re);
	--
	r:add(RotationElement:new(ABILITY_HW));

	r:SetName(name);
	r.locked = true;	
	self.rotations[r:GetName()] = r;
end

-- Construct the default high threat single-target rotation
local function createHighThreatSingleTargetRotation(self, name)
	local r = Rotation:new();	
	
	local re = RotationElement:new(ABILITY_SHOR);
	re.buffs = { [EFFECT_INQ]=true };
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_SHOR);
	re.buffs = { [EFFECT_SD]=true };
	r:add(re);
	--
	r:add(RotationElement:new(ABILITY_INQ));
	--
	r:add(RotationElement:new(ABILITY_CS));
	--
	r:add(RotationElement:new(ABILITY_AS));
	--
	r:add(RotationElement:new(ABILITY_HOW));
	--
	r:add(RotationElement:new(ABILITY_J));
	--
	local re = RotationElement:new(ABILITY_CON);
	re.manaMin = 75;
	r:add(re);
	--
	r:add(RotationElement:new(ABILITY_HW));

	r:SetName(name);
	r.locked = true;	
	self.rotations[r:GetName()] = r;
end

-- Construct the default AoE rotation
local function createAoERotation(self, name)
	local r = Rotation:new();	
	
	r:add(RotationElement:new(ABILITY_INQ));
	--
	r:add(RotationElement:new(ABILITY_HOR));
	--
	r:add(RotationElement:new(ABILITY_AS));
	--
	local re = RotationElement:new(ABILITY_J);
	re.manaMax = 50;
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_CON);
	re.manaMin = 75;
	r:add(re);
	--
	r:add(RotationElement:new(ABILITY_HW));
	--
	r:add(RotationElement:new(ABILITY_J));

	r:SetName(name);
	r.locked = true;	
	self.rotations[r:GetName()] = r;
end

-- Construct the high threat AoE rotation
local function createHighThreatAoERotation(self, name)
	local r = Rotation:new();	
	
	local re = RotationElement:new(ABILITY_INQ);
	re.buffs = { [EFFECT_INQ]=false };
	r:add(re);
	--
	r:add(RotationElement:new(ABILITY_HOR));
	--
	r:add(RotationElement:new(ABILITY_AS));
	--
	local re = RotationElement:new(ABILITY_J);
	re.manaMax = 50;
	r:add(re);
	--
	local re = RotationElement:new(ABILITY_CON);
	re.manaMin = 75;
	r:add(re);
	--
	r:add(RotationElement:new(ABILITY_HW));
	--
	r:add(RotationElement:new(ABILITY_J));

	r:SetName(name);
	r.locked = true;	
	self.rotations[r:GetName()] = r;
end

-- Initialize the instance
function ProtectionSpec:Init()
	self.state = new(ProtectionState);
	self.spellinfo = new(ProtectionSpellinfo);
	
	self.singleRotation = self:GetDefaultSingleRotation();
	self.aoeRotation = self:GetDefaultAOERotation();
	
	-- Create default rotations
	createOldSingleTargetRotation(self, DEFAULT_1_0);
	createNewSingleTargetRotation(self, DEFAULT_1_1);
	createAoERotation(self, DEFAULT_AOE);
	createHighThreatSingleTargetRotation(self, HIGH_THREAT_1_1);
	createHighThreatAoERotation(self, HIGH_THREAT_AOE);
end

-- Get the name of the class supported by this spec
function ProtectionSpec:GetClass()
	return "PALADIN";
end

-- Get the ID of the talent tree supported by this spec
function ProtectionSpec:GetTalentTree()
	return 2;		-- Protection
end

-- Get the description of this spec
function ProtectionSpec:GetDescription()
	return "Paladin (Protection)", "|cFFFF80C0Paladin|r (Protection)";
end

-- Get the label of the cost range input boxes
function ProtectionSpec:GetCostRangeLabel()
	return "% of base mana";
end

-- Get name of the default single-target rotation
function ProtectionSpec:GetDefaultSingleRotation()
	return DEFAULT_1_1;
end

-- Get name of the default AOE rotation
function ProtectionSpec:GetDefaultAOERotation()
	return DEFAULT_AOE;
end

-- Check whether this spec supports Holy Power
function ProtectionSpec:SupportsHolyPower()
	return true;
end

-- Add this spec to the list of supported specs
SpecList:Add(new(ProtectionSpec));
