Config = { };

-- Return the argument or the default value if the argument is nil
local function correct(x, default)
	if(x == nil) then
		return default;
	else
		return x;
	end
end

-- Get a value from the first table. Use the default value if it's not there.
-- Put that value into both tables.
local function copy(from, to, name, default)
	local x = from[name];
	if(x == nil) then
		x = default;
	else
		x = deepCopy(x);
	end;
	from[name] = x;
	to[name] = x;
end

-- Get all values from the first table. Use the defaults if needed and
-- put the values into both tables.
local function copyAll(from, to)
	-- General
	copy(from, to, "combatToggle", true);
	copy(from, to, "checkStance", true);
	copy(from, to, "locked", true);
	copy(from, to, "scale", 1.0);
	copy(from, to, "enabled", false);
	
	-- Display
	copy(from, to, "style", "default");
	copy(from, to, "fadeOut", false);
	copy(from, to, "blink", false);
	copy(from, to, "vengeanceEnabled", true);
	copy(from, to, "vengeanceStyle", "purple");
	copy(from, to, "showRotation", true);
	copy(from, to, "shiftSpeed", 1.0);
	copy(from, to, "shiftGCD", false);
	
	-- Sound
	copy(from, to, "sound", true);
	copy(from, to, "quietStart", true);
	copy(from, to, "reduceVolume", true);
	copy(from, to, "masterChannel", false);
	
	-- Rotation control
	copy(from, to, "priority", 300);
	copy(from, to, "singlePage", 0);
	copy(from, to, "aoePage", 0);
end

-- Load the configuration and set the default values
function Config:Load()
	if(AngeleDeiConfiguration == nil) then
		AngeleDeiConfiguration = { };
	end

	local c = new(Config);
	copyAll(AngeleDeiConfiguration, c);
	return c;
end

-- Save the configuration
function Config:Save()
	if(AngeleDeiConfiguration == nil) then
		AngeleDeiConfiguration = { };
	end

	copyAll(self, AngeleDeiConfiguration);
end
