-- This is a generic spec list. Individual specs should use the Add method to
-- register themselves with the addon
SpecList = {
	["list"] = { }
};

-- Add a spec to the list
function SpecList:Add(spec)
	self.list[#self.list+1] = new(spec);
	table.sort(self.list, function(a, b)
		return a:GetDescription() < b:GetDescription();
	end);
end

-- Find out if the player has a buff with the given ID
local function hasBuff(spellID)
	for i=1,40 do
		local id = select(9, UnitBuff("player", i));
		if(id == nil) then
			return false;
		elseif(id == spellID) then
			return true;
		end
	end
	return false;
end

-- Get the spec object matching the current character spec
function SpecList:GetActiveSpec(stanceRequired)
	local current = GetPrimaryTalentTree();
	local _, cls = UnitClass("player");
	for _,spec in ipairs(self.list) do
		if(spec:GetClass() == cls) and (spec:GetTalentTree() == current) then
			local stanceBuff = spec:GetRequiredStanceBuff();
			if(not stanceRequired) or (stanceBuff == nil) or (adBuffExpiration(stanceBuff)) then
				return spec;
			end
		end
	end
	
	-- Not found
	return nil;
end

-- Load all specs
function SpecList:LoadAllSpecs()
	for _,spec in ipairs(self.list) do
		spec:Load();
	end
end

-- Get the list of all available specs
function SpecList:GetAll()
	return self.list;
end