-- Create a deep copy of an arbitrary object
function deepCopy(this)
	local lookup_table = { };
	local function _copy(this)
		if type(this) ~= "table" then
			return this;
		elseif lookup_table[this] then
			return lookup_table[this];
		end
		local new_table = {};
		lookup_table[this] = new_table;
		for index, value in pairs(this) do
			new_table[_copy(index)] = _copy(value);
		end
		return setmetatable(new_table, getmetatable(this));
	end
	return _copy(this);
end

-- Create a deep copy of an arbitrary object and re-initialize it
function new(this)
	if(this == nil) then
		error("new(): argument cannot be nil");
	end
	
	local x = deepCopy(this);
	if(x.Init) then
		x:Init();
	end
	return x;
end

-- Get the buff expiration time
function adBuffExpiration(id)
	for i=1,64 do
		local name, _, _, _, _, _,expirationTime,_,_,_,spellID = UnitBuff("player", i);
		if(name == nil) then
			return nil;
		end
		if(spellID == id) then
			return expirationTime;
		end
	end
	return nil;
end

-- Get the rank of the given talent
function talentRank(spellID)
	-- Translate spell ID into spell name to avoid localization
	local name = GetSpellInfo(spellID);
	if(name == nil) then
		return 0;
	end
	
	local tabs = GetNumTalentTabs();
	for t=1,tabs do
	    local talents = GetNumTalents(t);
	    for i=1,talents do
	        local n, _, _, _, rank = GetTalentInfo(t, i);
	        if(n == name) then
	        	return rank;
	        end
	    end
	end

	return 0;
end

-- Check if we have the glyph with the given spellID (glyph types: 1=major, 2=minor, 3=prime)
function hasGlyph(targetSpellID, targetType)
	for i = 1, GetNumGlyphSockets() do
		local enabled, glyphType, glyphLocation, glyphSpellID, icon = GetGlyphSocketInfo(i);
		if(enabled) and (glyphSpellID) then
			if(glyphSpellID == targetSpellID) and (glyphType == targetType) then
				return true;
			end
		end
	end
	return false;
end

-- Find the index of the given value in a table
function indexOf(t, value, default)
	for i=1,#t do
		if(t[i] == value) then
			return i;
		end
	end
	
	return default;
end
